# fmt: off
from SpectrumCommon.Const.CS import *
from  SpectrumCommon.Const.CS import *
from SpectrumCommon.Modvars.GB.GBDefs import *

cs_bounds_dim = 'cs_bounds'
cs_bounds_indices = {
    CS_bAuto   : 'Auto',
    #  User results
    CS_bUser   : 'bUser',
    #  Estimate Bounds
    CS_bMin    : 'Min',
    CS_bMax    : 'Max',
    #  CSUA
    CS_bLow    : 'Low',
    CS_bMed    : 'Med',
    CS_bHigh   : 'High',
}
cs_age_dim = 'cs_age'
cs_age_indices = {
    CS_Birth       : 'Birth',
    CS_0t1months   : '0 to 1 months',
    CS_1t6months   : '1 to 6 months',
    CS_6t12months  : '6 to 12 months',
    CS_12t24months : '12 to 24 months',
    CS_24t60months : '24 to 60 months'
}


cs_work_year_dim = 'cs_work_year'
cs_work_year_indices = {'firstYear':0, 'finalYear':-1}

cs_costs_dim = 'cs_costs'
cs_costs_indices = {
    CS_Benefits : 'Benefits',
    CS_Costs    : 'Costs',
}

cs_factors_dim = 'cs_factors'
cs_factors_indices = {
    CS_SocialValue : 'Social value',
    CS_PercStillbirthsIncluded : 'Percentage of stillbirths included',
}

cs_cost_types_dim = 'cs_cost_types'
cs_cost_types_indices = {
    CS_InterventionCosts : 'Intervention costs',
    CS_ProgramCosts : 'Program costs',
    CS_LogisticsAndWastage : 'Logistics and wastage',
    CS_InfrastructureInvest : 'Infrastructure investment',
    CS_OtherHealthSysCosts : 'Other health system costs',
}

cs_benefit_types_dim = 'cs_benefit_types'
cs_benefit_types_indices = {
    CS_SocBenSB :   'Social benefit for stillbirths',
    CS_SocBenChild : 'Social benefit for child deaths',
    CS_SocBenMat :  'Social benefit for maternal deaths',
    CS_EarnSB :   'Economic benefit for stillbirths',
    CS_EarnChild :  'Economic benefit for child deaths',
    CS_EarnMat : 'Economic benefit for maternal deaths',
    CS_StuntReducAddEarn : 'Stunting reduction additional earnings',
}

cs_gdp_dim = 'cs_gdp'
cs_gdp_indices = {
    CS_PerCapita : 'Per capita GDP',
    CS_PerWorker : 'Per worker GDP',
}

cs_nutrition_dim = 'cs_nutrition'
cs_nutrition_indices = {
    CS_VitADefic   : 'Vitamin A deficiency',
    CS_ZincDefic   : 'Zinc deficiency',
    CS_CalcDefic   : 'Calcium deficiency',
    CS_FolDefic    : 'Folate deficiency',
    CS_IronDefic   : 'Iron deficiency',
    CS_VitB6Defic  : 'Vitamin B6 deficiency',
    CS_VitB12Defic : 'Vitamin B12 deficiency',
    CS_VitCDefic   : 'Vitamin C deficiency',
    CS_CoppDefic   : 'Copper deficiency',
    CS_MagDefic    : 'Magnesium deficiency',
    CS_NiaDefic    : 'Niacin deficiency',
    CS_PhosDefic   : 'Phosphorus deficiency',
    CS_RiboDefic   : 'Riboflavin deficiency',
    CS_ThiaDefic   : 'Thiamine deficiency',
}
    


cs_cause_dim= 'cs_cause'
cs_cause_indices = {
    CS_NNDiarr           : 'NNDiarr',
    CS_FirstNNDeath      : 'FirstNNDeath',
    CS_NNSepsis          : 'NNSepsis',
    CS_NNPneumonia       : 'NNPneumonia',
    CS_NNAsphyxia        : 'NNAsphyxia',
    CS_NNPreterm         : 'NNPreterm',
    CS_NNTetanus         : 'NNTetanus',
    CS_NNCongen          : 'NNCongen',
    CS_NNOther           : 'NNOther',
    CS_FinalStdNNDeath   : 'FinalStdNNDeath',
    CS_CustomNNCOD1      : 'CustomNNCOD1',
    CS_FirstCustNNDeath  : 'FirstCustNNDeath ',
    CS_CustomNNCOD2      : 'CustomNNCOD2',
    CS_CustomNNCOD3      : 'CustomNNCOD3',
    CS_CustomNNCOD4      : 'CustomNNCOD4',
    CS_FinalCustNNDeath  : 'FinalCustNNDeath',
    CS_FinalNNDeath      : 'FinalNNDeath',
    CS_Diarrhea          : 'Diarrhea',
    CS_FirstPNNDeath     : 'FirstPNNDeath',
    CS_Pneumonia         : 'Pneumonia',
    CS_Meningitis        : 'Meningitis',
    CS_Measles           : 'Measles',
    CS_Malaria           : 'Malaria',
    CS_Pertussis         : 'Pertussis',
    CS_AIDS              : 'AIDS',
    CS_Injury            : 'Injury',
    CS_OtherInfecDis     : 'OtherInfecDis',
    CS_OtherNCD          : 'OtherNCD',
    CS_FinalStdPNNDeath  : 'FinalStdPNNDeath',
    CS_MaxDefDeathCauses : 'MaxDefDeathCauses',
    CS_CustomPNNCOD1     : 'CustomPNNCOD1',
    CS_FirstCustPNNDeath : 'FirstCustPNNDeath',
    CS_CustomPNNCOD2     : 'CustomPNNCOD2',
    CS_CustomPNNCOD3     : 'CustomPNNCOD3',
    CS_CustomPNNCOD4     : 'CustomPNNCOD4',
}

cs_perc_covered_dim = 'cs_perc_covered'
cs_perc_covered_indices = {
    CS_0PercCov   : '0%',
    CS_5PercCov   : '5%',
    CS_10PercCov  : '10%',
    CS_15PercCov  : '15%',
    CS_20PercCov  : '20%',
    CS_25PercCov  : '25%',
    CS_30PercCov  : '30%',
    CS_35PercCov  : '35%',
    CS_40PercCov  : '40%',
    CS_45PercCov  : '45%',
    CS_50PercCov  : '50%',
    CS_55PercCov  : '55%',
    CS_60PercCov  : '60%',
    CS_65PercCov  : '65%',
    CS_70PercCov  : '70%',
    CS_75PercCov  : '75%',
    CS_80PercCov  : '80%',
    CS_85PercCov  : '85%',
    CS_90PercCov  : '90%',
    CS_95PercCov  : '95%',
    CS_100PercCov : '100%',
}
    
cs_mort_dim = 'cs_mort'
cs_mort_indices = {
    CS_NeonatalMR : 'Neonatal mortality rate',
    CS_InfantMR   : 'Infant mortality rate',
    CS_U5MR       : 'Under-5 mortality rate'
}
cs_diarr_path_dim = 'cs_diarr_path'
cs_diarr_path_indices = {
    CS_RotaPathDiar   : 'RotaPathDiar',
    CS_PathBDiar      : 'Path B Diarrea',
    CS_PathCDiar      : 'Path C Diarrea',
    CS_OtherPathsDiar : 'Other Paths Diarrea',
}
cs_fatality_dim = 'cs_fatality'
cs_fatality_indices = {
    CS_Distr_All   : 'Distribution All',
    CS_Distr_Sevr  : 'Distribution Severe',
    CS_Distr_Fatal : 'Distribution Fatal',
}

cs_mortality_age_dim = 'cs_mortality_age'
cs_mortality_age_indices = {
    
    CS_NeonatalMR : 'Neonatal MR',
    CS_InfantMR   : 'Infant MR',
    CS_U5MR       : 'Under-5 MR',
}
cs_death_cause_dim = 'cs_death_cause'
cs_death_cause_indices = {
    CS_DxC_Default    : 'Default',
    CS_DxC_Normalized : 'Normalized',
    CS_DxC_NoAIDS     : 'No AIDS',
}

cs_adol_age_dim = 'cs_adol_age'
cs_adol_age_indices = {
    CS_5yrs_9yrs   : '5  to 9 years',
    CS_10yrs_14yrs : '10 to 14 years',
    CS_15yrs_19yrs : '15 to 19 years',
}
cs_adol_causes_dim = 'cs_adol_causes'
cs_adol_causes_indices = {
    CS_Adol_HIV            : 'Adolecsent HIV',
    CS_Adol_Diarrhoeal     : 'Adolescent Diarrhoeal',
    CS_Adol_Measles        : 'Adolescent Measles',
    CS_Adol_Malaria        : 'Adolescent Malaria',
    CS_Adol_LRI            : 'Adolescent LRI',
    CS_Adol_TB             : 'Adolescent TB',
    CS_Adol_Maternal       : 'Adolescent Maternal',
    CS_Adol_OtherCMPN      : 'Adolescent Other CMPN',
    CS_Adol_Congenital     : 'Adolescent Congenital',
    CS_Adol_Neoplasm       : 'Adolescent Neoplasm',
    CS_Adol_Cardiovascular : 'Adolescent Cardiovascular',
    CS_Adol_Digestive      : 'Adolescent Digestive',
    CS_Adol_OtherNCD       : 'Adolescent Other NCD',
    CS_Adol_RTI            : 'Adolescent RTI',
    CS_Adol_Drowning       : 'Adolescent Drowning',
    CS_Adol_NatDis         : 'Adolescent NatDis',
    CS_Adol_InterpVio      : 'Adolescent Interpersonal Violence',
    CS_Adol_CollectVio     : 'Adolescent Collective Violence',
    CS_Adol_SelfHarm       : 'Adolescent Self Harm',
    CS_Adol_OtherInj       : 'Adolescent Other Injury',
}


cs_4t5Yr_dim = 'cs_4t5Yr'
cs_4t5Yr_indices = {
    CS_0t1Yr : '0  to 1 year',
    CS_1t2Yr : '1  to 2 years', 
    CS_2t3Yr : '2  to 3 years',
    CS_3t4Yr : '3  to 4 years', 
    CS_4t5Yr : '4  to 5 years'
}

cs_interv_dim = 'cs_interv'
cs_interv_indices = {}

cs_last_vacc_dim = 'cs_last_vacc'
cs_last_vacc_indices = {
    CS_BCG_Det         : 'BCG Det vaccine',
    CS_Polio_Det       : 'Polio Det vaccine',
    CS_DPT_Det         : 'DPT Det vaccine',
    CS_HiB_Det         : 'HiB Det vaccine',
    CS_HepB_Det        : 'HepB Det vaccine',
    CS_PCV_Det         : 'PCV Det vaccine',
    CS_Rota_Det        : 'Rota Det vaccine',
    CS_MenA_Det        : 'MenA Det vaccine',
    CS_Malaria_Det     : 'Malaria Det vaccine',
    CS_Meas_Det        : 'Meas Det vaccine',
    CS_VaccB_Det       : 'VaccB Det vaccine',
    CS_VaccC_Det       : 'VaccC Det vaccine',
    CS_VaccD_Det       : 'VaccD Det vaccine',
    CS_VaccE_Det       : 'VaccE Det vaccine',
    CS_VaccF_Det       : 'VaccF Det vaccine',
}

cs_vacc_dose_dim = 'cs_vacc_dose'
cs_vacc_dose_indices = {
    
    CS_VaccDoseZero   : 'Dose 0',
    CS_VaccDoseOne    : 'Dose 1',
    CS_VaccDoseTwo    : 'Dose 2',
    CS_VaccDoseThree  : 'Dose 3',
    CS_VaccBooster    : 'Dose booster',
}

cs_retro_years_dim = 'cs_retro_years'
cs_retro_years_indices = {}

cs_mat_death_dim = 'cs_max_death'
cs_mat_death_indices = {
    CS_Mat_AntepartHemorr    : 'Maternal AntepartHemorr',
    CS_Mat_MinDeathCauses    : 'Maternal MinDeathCauses',
    CS_Mat_IntrapartHemorr   : 'Maternal IntrapartHemorr',
    CS_Mat_PostpartHemorr    : 'Maternal PostpartHemorr',
    CS_Mat_HypertensiveDis   : 'Maternal HypertensiveDis',
    CS_Mat_Sepsis            : 'Maternal Sepsis',
    CS_Mat_Abortion          : 'Maternal Abortion',
    CS_Mat_Embolism          : 'Maternal Embolism',
    CS_Mat_OtherDirectCauses : 'Maternal OtherDirectCauses',
    CS_Mat_IndirectCauses    : 'Maternal IndirectCauses',
}

cs_abort_dim = 'cs_abort'
cs_abort_indices = {
    CS_TotalAbort  : 'Total abortions',
    CS_UnsafeAbort : 'Unsafe abortions',
    CS_SafeAbort   : 'Safe abortions',
}
cs_sbcauses_dim = 'cs_sbcauses'
cs_sbcauses_indices = {
    CS_SB_Antepartum  : 'Antepartum stillbirth',
    CS_SB_Intrapartum : 'Intrapartum stillbirth',
}

cs_perc_U5_mort_dim = 'cs_perc_U5_mort'
cs_perc_U5_mort_indices = {}

cs_std_dim = 'cs_std'
cs_std_indices = {
    CS_GT1STD : 'GT1STD',
    CS_1t2STD : '1t2STD',
    CS_2t3STD : '2t3STD',
    CS_GT3STD : 'GT3STD',
}
cs_discount_dim = 'cs_discount'
cs_discount_indices = {
    CS_3Perc  : '3%',
    CS_5Perc  : '5%',
    CS_10Perc : '10%',
}

cs_bf_type_dim= 'cs_bf_type'
cs_bf_type_indices = {
    CS_ExclusiveBF    : 'Exclusively breastfed',
    CS_PredominantBF  : 'Predominantly breastfed',
    CS_PartialBF      : 'Partially breastfed',
    CS_NotBF          : 'Not breastfed',
}

cs_latebf_dim = 'cs_latebf'
cs_latebf_indices = {
    CS_EarlyInitBF : 'Early initiation of breastfeeding',
    CS_LateInitBF  : 'Late initiation of breastfeeding',
}


cs_birth_term_dim = 'cs_birth_term'
cs_birth_term_indices = {
    CS_PreTermSGA  : 'Preterm SGA',
    CS_PreTermAGA  : 'Preterm AGA',
    CS_TermSGA     : 'Term SGA',
    CS_TermAGA     : 'Term AGA',
}

cs_birth_order_dim = 'cs_birth_order'
cs_birth_order_indices = {
    CS_FirstBirth         : 'First birth',
    CS_SecAndThirdBirths  : 'Second and third births',
    CS_GTThirdBirth       : 'Greater than third birth',
}

cs_mothers_age_dim = 'cs_mothers_age'
cs_mothers_age_indices = {
    CS_AllYrs       : 'All years',
    CS_LT18Yrs      : 'Less than 18 years',
    CS_18_34Yrs     : '18 to 34 years',
    CS_35_49Yrs     : '35 to 49 years',
}

    
CS_Efficacy                 = 1
CS_OddsRatio                = 1
CS_RelRisk                  = 1
CS_AffecFract               = 2
CS_AdjustedRR               = 3

    
cs_birth_interval_dim = 'cs_birth_interval'
cs_birth_interval_indices = {
    CS_FirstBirth         : 'First birth',
    CS_LT18Mths   : 'Less than 18 months',
    CS_18to23Mths : '18 to 23 months',
    CS_GE24Mths   : '24 months or greater',
}

cs_anemia_dim = 'cs_anemia'
cs_anemia_indices = {
    CS_Preg_Anemia              : 'Pregnant anemia',
    CS_NonPreg_Anemia           : 'Non-pregnant anemia',
    CS_Preg_IronDeficAnemia     : 'Pregnant iron deficiency anemia',
    CS_NonPreg_IronDeficAnemia  : 'Non-pregnant iron deficiency anemia',
}

cs_mat_anemia_dim = 'cs_mat_anemia'
cs_mat_anemia_indices = {
    CS_Mat_NotAnemic : 'Not anemic',
    CS_Mat_Anemic    : 'Anemic',
}

cs_bfpromotion_dim = 'cs_bfpromotion'
cs_bfpromotion_indices = {
    CS_BasicSanitation_DiarInterv         : 'Basic sanitation',
    CS_PointOfUseFilteredWater_DiarInterv : 'Point of use filtered water',
    CS_PipedWater_DiarInterv              : 'Piped water',
    CS_HandWashWSoap_DiarInterv           : 'Hand wash with soap',
    CS_HygDisposalStools_DiarInterv       : 'Hygienic disposal of stools',
    CS_ZincP_DiarInterv                   : 'Zinc supplementation',
    CS_VitAPTwoDoses_DiarInterv           : 'Vitamin A (two doses)',
    CS_RotavirusVacc_DiarInterv           : 'Rotavirus vaccine',
    CS_DiarVaccPathB_DiarInterv           : 'Diarrhea vaccine Path B',
    CS_DiarVaccPathC_DiarInterv           : 'Diarrhea vaccine Path C',
    CS_BFPromotion_DiarInterv             : 'BF promotion',
}
cs_server_diarr_dim = 'cs_server_diarr'
cs_server_diarr_indices = {
    CS_RotavirusVacc_SevDiarInterv  : 'Rotavirus vaccine',
}
cs_pnuem_dim = 'cs_pnuem'
cs_pnuem_indices = {
    CS_HibVacc_PneumInterv   : 'HIB vaccine',
    CS_PneumVacc_PneumInterv : 'Pneumococcal vaccine',
    CS_ZincP_PneumInterv     : 'Zinc supplementation',
}

cs_menin_dim = 'cs_menin'
cs_menin_indices = {
    CS_HibVacc_MeningInterv   : 'HIB vaccine',
    CS_PneumVacc_MeningInterv : 'Pneumococcal vaccine',
    CS_MeningA_MeningInterv   : 'Meningitis A vaccine',
}
cs_pnuem_path_dim= 'cs_pnuem_path'
cs_pnuem_path_indices = {
    CS_HibPathPneumVT     : 'Hib Path Pneum VT',
    CS_PneumocPathPneum   : 'Pneumoc Path Pneum',
    CS_PneumocPathPneumVT : 'Pneumoc Path Pneum VT',
    CS_OtherPathsPneum    : 'Other Paths Pneum',
}
cs_menin_path_dim= 'cs_menin_path'
cs_menin_path_indices = {
    CS_HibPathMeninVT       : 'Hib Path Menin VT',
    CS_PneumocPathMenin     : 'Pneumoc Path Menin',
    CS_PneumocPathMeninVT   : 'Pneumoc Path Menin VT',
    CS_MeningitPathMenin    : 'Meningit Path Menin',
    CS_MeningitAPathMeninVT : 'Meningit A Path Menin VT',
    CS_OtherPathsMenin      : 'Other Paths Menin',
}

cs_data_dim = 'cs_data'
cs_data_indices = {
    
    CS_Default : 'Defaults',
    CS_Data    : 'Editable user data',
}
cs_affect_fract_dim = 'cs_affect_fract'
cs_affect_fract_indices = {
    CS_RelRisk     : 'Relative risk or odds ratio or efficacy',
    CS_AffecFract  : 'Affected fraction',
    CS_AdjustedRR  : 'Adjusted relative risk',
} 
cs_birth_outcomes_dim = 'cs_birth_outcomes'
cs_birth_outcomes_indices = {
    
    CS_IPTp               : 'Intermittent preventive treatment in pregnancy',
    CS_FirstIntervForBO   : 'First intervention for birth outcomes',
    CS_BalEnergy          : 'Balanced energy',
    CS_MultMicroLowBMI    : 'Multimicronutrient supplementation for low BMI',
    CS_MultMicroNormBMI   : 'Multimicronutrient supplementation for normal BMI',
    CS_CalciumSupp        : 'Calcium supplementation',
    CS_FolicAcidFort      : 'Folic acid fortification',
    CS_ZincFort           : 'Zinc fortification',
    CS_SyphilisTreat      : 'Syphilis treatment',
    CS_IronFolateSupp     : 'Iron and folate supplementation',
    CS_ZincSuppInPreg     : 'Zinc supplementation in pregnancy',
    CS_TreatOfBacteriuria : 'Treatment of bacteriuria',
    CS_Omeg3FASupp        : 'Omega-3 fatty acid supplementation',
    CS_AspirinLowDose     : 'Low-dose aspirin',
    CS_Progesterone       : 'Progesterone',
    CS_StopSmokingEduc    : 'Stop smoking education',
    CS_SpareBOInterv1     : 'Spare birth outcomes intervention 1',
    CS_SpareBOInterv2     : 'Spare birth outcomes intervention 2',
    CS_SpareBOInterv3     : 'Spare birth outcomes intervention 3',
    CS_SpareBOInterv4     : 'Spare birth outcomes intervention 4',
    CS_FoodInsecBES       : 'Food insecurity basic education and support',
    CS_ITN                : 'Insecticide-treated nets',
    CS_FirstBOMax         : 'First birth outcome maximum',
    CS_AgeAndBirthOrder   : 'Age and birth order',
    CS_BirthInterval      : 'Birth interval',
}

cs_stunting_dim = 'cs_stunting'
cs_stunting_indices = {
    CS_LowBirthWeight_TermAGA    : 'Low birth weight term AGA',
    CS_LowBirthWeight_TermSGA    : 'Low birth weight term SGA',
    CS_LowBirthWeight_PreTermAGA : 'Low birth weight preterm AGA',
    CS_LowBirthWeight_PreTermSGA : 'Low birth weight preterm SGA',
    CS_NotStuntedPrevAge         : 'Not stunted previous age',
    CS_StuntedPrevAge            : 'Stunted previous age',
    CS_FoodSecureWPromo          : 'Food secure with promotion',
    CS_FoodSecureWOPromo         : 'Food secure without promotion',
    CS_InsecureWPromoSupp        : 'Insecure with promotion and supplementation',
    CS_InsecureWOPromoSupp       : 'Insecure without promotion and supplementation',
    CS_ImpDiarrPerEpisode        : 'Impact of diarrhea per episode', 
    CS_ZincSupp                  : 'Zinc supplementation',
    CS_NotZincSupp               : 'Not zinc supplementation',
    CS_ImpactsOnStuntingFinal    : 'Impacts on stunting final',
}

cs_wasting_dim = 'cs_wasting'
cs_wasting_indices = {
    CS_ImpactsOnWaste_FractionChildrenMovModWastWithTreat : 'Fraction of children moving from moderate to severe wasting with treatment',
    CS_ImpactsOnWaste_FractionChildrenMovSevWastWithTreat : 'Fraction of children moving from severe wasting to moderate wasting with treatment',
}

cs_preg_dim = 'cs_preg'
cs_preg_indices = {
    CS_Preg    : 'Pregnant',
    CS_NonPreg : 'Not pregnant',
}

cs_impacts_mat_anemia_dim = 'cs_impacts_mat_anemia'
cs_impacts_mat_anemia_indices = {
    CS_IronFolatePW_MatAnemia : 'Iron and folate supplementation in pregnancy',
    CS_MMicroSuppPW_MatAnemia : 'Multimicronutrient supplementation in pregnancy',
    CS_IronFortNPW_MatAnemia  : 'Iron fortification in non-pregnant in pregnancy',
    CS_IPTP_MatAnemia         : 'Intermittent preventive treatment in pregnancy',
    CS_ITN_MatAnemia          : 'Insecticide-treated nets in pregnancy',
}

cs_impacts_bf_dim = 'cs_impacts_bf'
cs_impacts_bf_indices = {
    CS_ImpactPromoBF      : 'Impact on breast feeding promo',
    CS_ImpactPromoEarlyBF : 'Early impact on breast feeding promo',
}

cs_bf_promo_dim = 'cs_bf_promo'
cs_bf_promo_indices = {
    CS_BFPromoHealthSystem : 'Health system',
    CS_BFPromoHomeComm     : 'Home and community',
    CS_BFPromoBoth         : 'Both',
}   

cs_final_interv_for_fi_for_calcs_dim = 'cs_final_interv_for_fi_for_calcs'
cs_final_interv_for_fi_for_calcs_indices = {
    CS_ConditionalCashTransfer  : 'Conditional cash transfer',
    CS_FoodAssistanceToFamilies : 'Food assistance to families',
    CS_DirectSupport            : 'Direct support',
    CS_FoodShock                : 'Food shock',
}

# Originally created all these wrappers because
# the cs modvars appeared to have multiple fields in the code.
# However, it seems all the extra fields are moved to meta modvars 
# at the end of CSModvars in Meta_MV_Init

def cs_bool_shape(d):
    return  gb_bool_shape(d)

def cs_int_shape(d):
    return  gb_int_shape(d)

def cs_float_shape(d):
    return  gb_float_shape(d)

def cs_str_shape(d):
    return gb_str_shape(d)

def cs_year_shape(d):
    return gb_shape(d, '2D float', dim=[cs_bounds_dim, gb_year_dim], indices=[cs_bounds_indices, gb_year_indices])

def cs_age_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_age_indices, gb_year_indices])

def cs_cause_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_cause_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_cause_indices, cs_age_indices, gb_year_indices])

def cs_diarr_paths_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_diarr_path_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_diarr_path_indices, cs_age_indices, gb_year_indices])

def cs_diarr_paths_agegroup_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_diarr_path_dim, cs_4t5Yr_dim, gb_year_dim], indices=[cs_bounds_indices, cs_diarr_path_indices, cs_4t5Yr_indices, gb_year_indices])

def cs_diarr_paths_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_diarr_path_dim,  gb_year_dim], indices=[cs_bounds_indices, cs_diarr_path_indices, gb_year_indices])


def cs_deaths_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_cause_dim, gb_year_dim], indices=[cs_bounds_indices, cs_cause_indices, gb_year_indices])

def cs_interv_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_interv_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_age_indices, gb_year_indices])


def cs_interv_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_interv_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, gb_year_indices])

def cs_interv_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_interv_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_age_indices, gb_year_indices])

def cs_interv_cause_age_year_shape(d):
    return gb_shape(d, '5D float', dim=[cs_bounds_dim, cs_interv_dim, cs_cause_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_cause_indices, cs_age_indices, gb_year_indices])

def cs_mat_death_causes_by_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_mat_death_dim, gb_year_dim], indices=[cs_bounds_indices, cs_mat_death_indices, gb_year_indices])


def cs_mat_deaths_prev_by_inter_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_interv_dim, cs_mat_death_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_mat_death_indices, gb_year_indices])


def cs_num_abortions_shape(d):
    return gb_shape(d, '3D float', dim=[CS_bHigh+1, cs_abort_dim, gb_year_dim], indices=[cs_bounds_indices, cs_abort_indices, gb_year_indices])

def cs_sbcauses_year(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_sbcauses_dim, gb_year_dim], indices=[cs_bounds_indices, cs_sbcauses_indices, gb_year_indices])

def cs_interv_sbcauses_year(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_interv_dim, cs_sbcauses_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_sbcauses_indices, gb_year_indices])

def cs_red_mort_interv_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_perc_U5_mort_dim, cs_interv_dim], indices=[cs_bounds_indices, cs_perc_U5_mort_indices, cs_interv_indices, cs_sbcauses_indices, gb_year_indices])

def cs_interv_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_interv_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_sbcauses_indices, gb_year_indices])

def cs_subop_bo_avert_by_inter_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_interv_dim, cs_std_dim, gb_year_dim], indices=[cs_bounds_indices, cs_interv_indices, cs_std_indices, gb_year_indices])

def cs_age_shape(d):
    return gb_shape(d, '2D float', dim=[cs_bounds_dim, cs_age_dim], indices=[cs_bounds_indices, cs_age_indices])

def cs_std_age_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_std_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_std_indices, cs_age_indices, gb_year_indices])

def cs_discount_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_discount_dim, gb_year_dim], indices=[cs_bounds_indices, cs_discount_indices, gb_year_indices])


def cs_bf_prev_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_bf_type_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_bf_type_indices, cs_age_indices, gb_year_indices])

def cs_birth_term_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_birth_term_dim, gb_year_dim], indices=[cs_bounds_indices, cs_birth_term_indices, gb_year_indices])

def cs_anemia_by_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_anemia_dim, gb_year_dim], indices=[cs_bounds_indices, cs_anemia_indices, gb_year_indices])

def cs_bfpromotion_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_bfpromotion_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_bfpromotion_dim, cs_age_indices, gb_year_indices])

def cs_pneum_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_pnuem_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_pnuem_indices, cs_age_indices, gb_year_indices])

def cs_menin_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_menin_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_menin_indices, cs_age_indices, gb_year_indices])


def cs_pneum_path_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_pnuem_path_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_pnuem_path_indices, cs_age_indices, gb_year_indices])


def cs_menin_path_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_menin_path_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_menin_path_indices, cs_age_indices, gb_year_indices])

def cs_pneum_path_agegroup_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_pnuem_path_dim, cs_4t5Yr_dim, gb_year_dim], indices=[cs_bounds_indices, cs_pnuem_path_indices, cs_4t5Yr_indices, gb_year_indices])


def cs_menin_path_agegroup_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_menin_path_dim, cs_4t5Yr_dim, gb_year_dim], indices=[cs_bounds_indices, cs_menin_path_indices, cs_4t5Yr_indices, gb_year_indices])

def cs_pneum_path_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_pnuem_path_dim, gb_year_dim], indices=[cs_bounds_indices, cs_pnuem_path_indices, gb_year_indices])


def cs_menin_path_year_shape(d):
    return gb_shape(d, '3D float', dim=[cs_bounds_dim, cs_menin_path_dim, gb_year_dim], indices=[cs_bounds_indices, cs_menin_path_indices, gb_year_indices])

def cs_vacc_age_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices,  cs_age_indices, gb_year_indices])

def cs_vacc_dose_agegroup_year_shape(d):
    return gb_shape(d, '5D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_vacc_dose_dim, cs_4t5Yr_dim, gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices,  cs_4t5Yr_indices, gb_year_indices])

def cs_vacc_dose_year_shape(d):
    return gb_shape(d, '4D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_vacc_dose_dim,  gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices,  gb_year_indices])

cs_age_cohort_dim = 'cs_age_cohort'
cs_age_cohort_indices = {}
def cs_agecohort_shape(d):
    return gb_shape(d, '1D float', dim=[cs_age_cohort_dim], indices=[cs_age_cohort_indices])


CS_modvar_shapes = {
        CS_DebugTag                              : gb_shape('Debug flag', '1D float'),
        # Subnational Wizard
        CS_SubnatWizProjTag                      : cs_bool_shape('Subnational wizard flag'),
        CS_SubnatRegionTag                       : cs_str_shape('Subnational region'),
        CS_FirstTimeEnteringCoverageTag          : cs_bool_shape('First time entering coverage flag'),
        CS_SubnatDefDataMstIDsetTag              : gb_shape('Subnational default mstID set', '1D int'),
        CS_SubnatDataSourceArrayTag              : gb_shape('Subnational source array', 'dict'),

        CS_DataSourceTag	                     : cs_str_shape('Data source'),
        
        # Meta data
        CS_IDMapTag                              : gb_shape('ID map', 'dict', notes='Map of IDs'),
        CS_FinalIndicesTag                       : gb_shape('Final indices constants', 'dict', notes='Max constants for indices'),
        
        # Intervention info
        CS_NumIntervsTag                         : cs_int_shape('Number of interventions'),
        CS_IVInfoTag                             : gb_shape('Intervention information by intervention id', 'dict'),   
        CS_IVGroupInfoTag                        : gb_shape('Intervention group information by group id', 'dict'),      
        CS_IVSubgroupInfoTag                     : gb_shape('Intervention subgroup information by subgroup id', 'dict'),      
        
        # General
        CS_DevModeTag                            : cs_bool_shape('Developer mode'),  
        CS_FirstYearTag                          : cs_int_shape('First year'),   
        CS_FinalYearTag                          : cs_int_shape('Final year'),  
        CS_ProjValidTag	                         : cs_bool_shape('projection valid flag'),
        CS_ProjValidDateTag	                     : cs_str_shape('projection valid date'),

        # Controls
        CS_CovBYTag                              : gb_int_shape('Coverage by year'),
        CS_MenARecommendedTag                    : cs_bool_shape('MenARecommended'),
        CS_IPTpCBCheckedTag                      : cs_bool_shape('IPTpCBChecked'),
        CS_HaveCalculatedChDeathsByCauseTag      : cs_bool_shape('Have calculated child deaths by cause flag'),                     
        CS_FirstYearAtWorkTag                    : cs_int_shape('First year at work'),
        CS_LastYearAtWorkTag                     : cs_int_shape('Last year at work'),
        CS_AbortCalcFamPlanTag                   : cs_bool_shape('Abortions calculated for family planning flag'),  
        CS_CovEntryModeTag                       : cs_int_shape('Coverage entry mode'),      
        CS_CalcCovFromInputsTag                  : gb_shape('Calculation coverage from inputs',  '1D int'),
        CS_EnterBFImproveDataTag                 : cs_int_shape('Enter BF improvement data'),
        CS_BFPromoCBsTag                         : gb_shape('Breast feeding promo CB',  '1D bool'),
        CS_StuntMatrixOnTag                      : cs_bool_shape('Stunt matrix on'),  
        CS_WasteMatrixOnTag                      : cs_bool_shape('Waste matrix on'),  
        CS_UseDetailedStuntMatrixTag             : cs_bool_shape('Use detailed stunt matrix'),  
        CS_UseDetailedWasteMatrixTag             : cs_bool_shape('Use detailed waste matrix'),  
        CS_FertilityRisksMatrixOnTag             : cs_bool_shape('Fertility risks matrix on'),  
        CS_DirectEntryFITag                      : cs_bool_shape('Direct entry FI flag'),  
        CS_SelectedIVSetTag                      : gb_shape('Selected IV set', '1D int'),

        # Health status
        CS_NutrDeficTag                          : gb_shape('Nutritional deficiency', '1D float',
                                                            dim=[cs_nutrition_dim],
                                                            indices=[cs_nutrition_indices]),
        
        CS_VitADefInPWTag                        : cs_float_shape('Vitamin A deficiency in pregnant women'),
        CS_StatusAtBirthTag                      : gb_shape('Nutritional deficiency', '1D float',
                                                            dim=[cs_birth_term_dim],
                                                            indices=[cs_birth_term_indices]),
        CS_DiarrIncTag                           : cs_agecohort_shape('Diarrhea incidence'),
        CS_SevereDiarrIncTag                     : cs_agecohort_shape('Severe diarrhea incidence'),
        CS_PneumIncTag                           : cs_agecohort_shape('Pneumonia incidence'),
        CS_MeninIncTag                           : cs_agecohort_shape('Meningitis incidence'),
        CS_MaternalAnemiaTag                     : gb_shape('Nutritional deficiency', '1D float',
                                                            dim=[cs_mat_anemia_dim],
                                                            indices=[cs_mat_anemia_indices]),
        CS_PercWom15t49LowBMITag                 : cs_float_shape('Percentage women 15 to 49 with low BMI'),
        CS_SyphilisAmongWRATag                   : cs_float_shape('Syphilis among Women of Reproductive Age'),
        CS_PercExposedFalciparumTag              : cs_float_shape('Percentage exposed to falciparum malaria'),
        CS_PercWomenWithPrevPreEclampTag         : cs_float_shape('Percentage with previous pre-eclampsia'),
        CS_PercWomenWithChronicHypertensionTag   : cs_float_shape('Percentage with chronic hypertension'),
        CS_FolicAcidDeficPrevTag                 : cs_float_shape('Folic acid deficiency prevalence'),
        CS_CongenitalDueToNTDsAFTag              : cs_float_shape('Congenital due to NTDs Affected Fraction'),
        CS_StuntDistrTag                         : gb_shape('Wasting distribution', '3D float',
                                                            dim=[cs_std_dim, cs_age_dim, gb_year_dim],
                                                            indices=[cs_std_indices, cs_age_indices, gb_year_indices]),
        CS_WasteDistrTag                         : gb_shape('Wasting distribution', '1D float',
                                                            dim=[cs_std_dim, cs_age_dim, gb_year_dim],
                                                            indices=[cs_std_indices, cs_age_indices, gb_year_indices]),
        CS_SingleStuntDistrTag                   : gb_shape('Single stunting distribution', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_SingleWasteDistrTag                   : gb_shape('Single wasting distribution', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_DiarrPathTag	                         : gb_shape('Diarrhea Path', '3D float',
                                                            dim=[cs_fatality_dim, cs_diarr_path_dim, cs_age_dim],
                                                            indices=[cs_fatality_indices, cs_diarr_path_indices, cs_age_indices]),
        CS_PneumPathTag                          : gb_shape('Diarrhea Path', '3D float',
                                                            dim=[cs_fatality_dim, cs_pnuem_path_dim, cs_age_dim],
                                                            indices=[cs_fatality_indices, cs_pnuem_path_indices, cs_age_indices]),
        CS_MeninPathTag	                         : gb_shape('Diarrhea Path', '3D float',
                                                            dim=[cs_fatality_dim, cs_menin_path_dim, cs_age_dim],
                                                            indices=[cs_fatality_indices, cs_menin_path_indices, cs_age_indices]),
                                                            
        CS_BaselineMortTag                       : gb_shape('Diarrhea Path', '1D float',
                                                            dim=[cs_mortality_age_dim],
                                                            indices=[cs_mortality_age_indices]),
            
        CS_PercDthByCauseTag	                 : gb_shape('Percent death by cause', '3D float',
                                                            dim=[cs_death_cause_dim, cs_cause_dim, cs_age_dim],
                                                            indices=[cs_death_cause_indices, cs_cause_dim, cs_age_indices]),
            
        CS_AdolMortRatioTag                      : gb_shape('Adolescent mortality ratio', '2D float',
                                                            dim=[gb_sex_dim, cs_adol_age_dim],
                                                            indices=[gb_sex_indices, cs_adol_age_indices]),
        CS_PercAdolDeathsByCauseTag	             : gb_shape('Adolescent mortality ratio', '3D float',
                                                            dim=[cs_adol_causes_dim, gb_sex_dim, cs_adol_age_dim],
                                                            indices=[cs_adol_causes_indices, gb_sex_indices, cs_adol_age_indices]),
        
        CS_MatMortRatioTag                       : cs_float_shape('Maternal mortality ratio'),
        CS_PercMatDthByCauseTag                  : gb_shape('Maternal deaths by cause', '1D float',
                                                            dim = [cs_mat_death_dim],
                                                            indices = [cs_mat_death_indices]),
        CS_PercPregEndAbortTag                   : cs_float_shape('Percentage of pregnancies ending in abortion'),
        CS_AbortIncRatioTag                      : gb_shape('Abortion incidence ratio', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_SBRateTag                             : cs_float_shape('Stillbirth rate'),
        CS_SBCausesTag                           : gb_shape('Still birth causes', '1D float',
                                                            dim=[cs_sbcauses_dim],
                                                            indices=[cs_sbcauses_indices]),
        CS_PerCapitaIncTag                       : gb_shape('Per capita inc', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_FoodInsecureTag                       : gb_shape('Food insecurity', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_AvgHouseholdSizeTag	                 : gb_shape('Average household size', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_IncludeCostForBCRTag                  : gb_shape('Include costs for BCR', '1D bool',
                                                            dim=[cs_cost_types_dim],
                                                            indices=[cs_cost_types_indices]),
                                                            
        CS_IncludeBenefitForBCRTag               : gb_shape('Include costs for BCR', '1D bool',
                                                            dim=[cs_benefit_types_dim],
                                                            indices=[cs_benefit_types_indices]),
        CS_BaselineGDPTag                        : gb_shape('Include costs for BCR', '1D bool',
                                                            dim=[cs_gdp_dim],
                                                            indices=[cs_gdp_indices]),
        CS_AnnualPercGrowthRateTag	             : gb_shape('Annual percent growth rate', '1D float',
                                                            dim=[cs_work_year_dim],
                                                            indices=[cs_work_year_indices]),
        CS_LaborForceParRateTag	                 : gb_shape('Labor force par rate', '1D float',
                                                            dim=[cs_work_year_dim],
                                                            indices=[cs_work_year_indices]),
        CS_FemaleLaborForceParRateTag	         : gb_shape('Female labor force par rate', '1D float',
                                                            dim=[cs_work_year_dim],
                                                            indices=[cs_work_year_indices]),
    
        CS_PercGDPAllocWageTag	                 : gb_shape('Percent GDP alloc wage', '1D float',
                                                            dim=[cs_work_year_dim],
                                                            indices=[cs_work_year_indices]),
    
    
        CS_DiscountRatesTag                      : gb_shape('Discount rates', '1D float',
                                                            dim=[cs_costs_dim],
                                                            indices=[cs_costs_indices]),
    
        CS_FactorsTag                            : gb_shape('Factors', '1D float',
                                                            dim=[cs_factors_dim],
                                                            indices=[cs_factors_indices]),

        # Coverage
        CS_CoverageTag                           : gb_shape('Coverage', '2D float',
                                                            dim=[ cs_interv_dim, gb_year_dim],
                                                            indices=[cs_interv_indices, gb_year_indices]),
        CS_BFPrevTag                             : gb_shape('Breast feeding prevelance', '3D float',
                                                            dim=[ cs_bf_type_dim, cs_age_dim, gb_year_dim],
                                                            indices=[cs_bf_type_indices, cs_age_indices, gb_year_indices]),
        CS_EarlyInitBFPrevTag                    : gb_shape('Early breast feeding prevelance', '2D float',
                                                            dim=[ cs_age_dim, gb_year_dim],
                                                            indices=[cs_age_indices, gb_year_indices]),
        CS_RoutDetVaccCovTag                     : gb_shape('Routa vaccine coverage', '3D float',
                                                            dim=[ cs_last_vacc_dim, cs_vacc_dose_dim, gb_year_dim],
                                                            indices=[cs_last_vacc_indices, cs_vacc_dose_indices, gb_year_indices]),
        CS_PentaDetVaccCovTag                    : gb_shape('Penta vaccine coverage', '1D float',
                                                            dim=[gb_year_dim],
                                                            indices=[gb_year_indices]),
        CS_VaccineSuppPercTag                    : gb_shape('Vaccine Supp percent', '3D float',
                                                            dim=[cs_last_vacc_dim, cs_age_dim, gb_year_dim],
                                                            indices=[cs_last_vacc_indices, cs_age_indices, gb_year_indices]),
        CS_AgeAndBirthOrderMatrixTag	         : gb_shape('Age and birth order matrix', '3D float',
                                                            dim=[cs_mothers_age_dim, cs_birth_order_dim, gb_year_dim],
                                                            indices=[cs_mothers_age_indices, cs_birth_order_indices, gb_year_indices]),
        CS_BirthIntervalsMatrixTag               : gb_shape('Birth interval matrix', '2D float',
                                                            dim=[cs_birth_interval_dim, gb_year_dim],
                                                            indices=[cs_birth_interval_dim, gb_year_indices]),
        # Maternal, Stillbirth, and Child Efficacy
        CS_MatEffTag                             : gb_shape('Maternal efficacy', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_interv_dim, cs_mat_death_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_interv_indices, cs_mat_death_indices]),

        CS_SBEffTag                              : gb_shape('Still birth efficacy', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_interv_dim, cs_sbcauses_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_interv_indices, cs_sbcauses_indices]),

        CS_ChildEffTag                           : gb_shape('Still birth efficacy', '5D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_interv_dim, cs_cause_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_interv_indices, cs_cause_indices, cs_age_indices]),

        CS_DetVaccEffTag                         : gb_shape('Still birth efficacy', '6D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_last_vacc_dim, cs_cause_dim, cs_vacc_dose_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_last_vacc_indices, cs_cause_indices, cs_vacc_dose_indices, cs_age_indices]),


        # Effectiveness of interventions on incidence
        CS_DiarrIncReductionTag                  : gb_shape('Diarrea incidence intervention effectiveness', '4D float',
                                                            dim=[cs_data_dim, cs_bfpromotion_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_bfpromotion_indices ,cs_affect_fract_indices, cs_age_indices]),

        CS_SevDiarrIncReductionTag               : gb_shape('Severe diarrea incidence intervention effectiveness', '4D float',
                                                            dim=[cs_data_dim, cs_server_diarr_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_server_diarr_indices, cs_affect_fract_indices, cs_age_indices]),
        
        CS_PneumIncReductionTag                  : gb_shape('Pneumonia incidence intervention effectiveness', '4D float',
                                                            dim=[cs_data_dim, cs_pnuem_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_pnuem_dim,cs_affect_fract_indices, cs_age_indices]),
        
        CS_MeninIncReductionTag                  : gb_shape('Meningitis incidence intervention effectiveness', '4D float',
                                                            dim=[cs_data_dim, cs_menin_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_menin_indices, cs_affect_fract_indices, cs_age_indices]),

        
        CS_RelRisksDiarrTag                      : gb_shape('Relative risks of diarrhea', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_bf_type_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_bf_type_indices, cs_age_indices]),

        CS_RelRisksPneumTag          	         : gb_shape('Relative risks of pneumonia', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_bf_type_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_bf_type_indices, cs_age_indices]),
        CS_RelRisksMeninTag	                     : gb_shape('Relative risks of meningitis', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_bf_type_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_bf_type_indices, cs_age_indices]),

        # Herd effectiveness of vaccines
        CS_HerdVaccEffTag                        : gb_shape('Herd vaccine effectiveness', '5D float',
                                                            dim=[cs_data_dim, cs_last_vacc_dim, cs_cause_dim, cs_perc_covered_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_last_vacc_indices, cs_cause_indices, cs_perc_covered_indices, cs_age_indices]),


        # Impact on food security
        CS_ImpOnFoodSecurityTag                  :  gb_shape('Impact on food security', '3D float',
                                                                                                             dim = [cs_data_dim, cs_affect_fract_dim, cs_final_interv_for_fi_for_calcs_dim],
                                                                                                             indices = [cs_data_indices, cs_affect_fract_indices, cs_final_interv_for_fi_for_calcs_indices]),

        # Effectiveness of nutrition interventions
        CS_AgeAndBirthOrderTag                   : gb_shape('Age and birth order', '5D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_age_dim, cs_birth_order_dim,  cs_birth_term_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_age_indices, cs_birth_order_indices, cs_birth_term_indices]),
                                                            
                                                        
        CS_BirthIntervalsTag	                 : gb_shape('Birth intervals', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_birth_interval_dim, cs_birth_term_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_birth_interval_indices, cs_birth_term_indices]),
       
        CS_IntervForBOTag                        : gb_shape('Intervention for birth order', '4D float',
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_birth_outcomes_dim, cs_birth_term_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_birth_outcomes_indices, cs_birth_term_indices]),

        CS_ImpactOnStuntTag                      : gb_shape('Impact on stunting', '4D float',   
                                                            dim=[cs_data_dim, cs_stunting_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_stunting_indices, cs_affect_fract_indices,  cs_age_indices]),
        
        CS_ImpactOnWasteTag                      : gb_shape('Impact on wasting', '4D float',   
                                                            dim=[cs_data_dim, cs_wasting_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_wasting_indices, cs_affect_fract_indices,  cs_age_indices]),

        
        CS_ImpactOnMatAnemiaTag  	             : gb_shape('Impact on wasting', '4D float',   
                                                            dim=[cs_data_dim, cs_preg_dim,  cs_impacts_mat_anemia_dim, cs_affect_fract_dim],
                                                            indices=[cs_data_indices, cs_preg_indices, cs_impacts_mat_anemia_indices, cs_affect_fract_indices]), 

        
        CS_ImpactBFPromoTag                      :  gb_shape('Impact on breast feeding promo', '5D float',   
                                                            dim=[cs_data_dim, cs_impacts_bf_dim, cs_bf_promo_dim, cs_affect_fract_dim,  cs_age_dim],
                                                            indices=[cs_data_indices, cs_impacts_bf_indices, cs_bf_promo_indices, cs_affect_fract_indices,  cs_age_indices]),
        # 'value'     : np.zeros((CS_Data+1, CS_ImpactPromoEarlyBF+1, CS_BFPromoBoth+1, CS_AffecFract+1, CS_24t60months+1)).tolist(),
        
        CS_KMConBFTag                            :  gb_shape('KMC on Breast feeding', '3D float',   
                                                            dim=[cs_data_dim, cs_affect_fract_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices, cs_age_indices]),
        CS_LBWFTag                               :  gb_shape('LBWF', '3D float',   
                                                            dim=[cs_data_dim, cs_birth_term_dim],
                                                            indices=[cs_data_indices, cs_birth_term_indices]),
        CS_ReducStuntTag                         : gb_shape('reduction on stunting', '2D float',   
                                                            dim=[cs_data_dim, cs_affect_fract_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices]),
        CS_EducAttainTag                         : gb_shape('Education attainment', '2D float',   
                                                            dim=[cs_data_dim, cs_affect_fract_dim],
                                                            indices=[cs_data_indices, cs_affect_fract_indices]),

        # Impact of under-nutrition on mortality
        CS_RelRisksStuntTag                      : gb_shape('Relative risks of stunting', '5D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_affect_fract_dim, cs_std_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_affect_fract_indices, cs_std_indices, cs_age_indices]),

        CS_RelRisksWasteTag                      : gb_shape('Relative risks of wasting', '3D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_birth_term_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_birth_term_indices]),
        #  np.zeros((CS_Data+1, CS_MaxDeathCauses+1, CS_TermAGA+1)).tolist(),
         
        CS_RelRisksIUGR_LBWTag	                 : gb_shape('Relative risks of IUGR and low birth weight', '5D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_affect_fract_dim, cs_latebf_dim, cs_bf_type_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_affect_fract_indices, cs_latebf_indices, cs_bf_type_indices]),
        # np.zeros((CS_Data+1, CS_MaxDeathCauses+1, CS_AffecFract+1, CS_LateInitBF + 1, CS_NotBF + 1)).tolist(),
        
        CS_ImpactBFOnMortNeoNatTag               : gb_shape('Impact of breastfeeding on neonatal mortality', '5D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_affect_fract_dim, cs_latebf_dim, cs_bf_type_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_affect_fract_indices, cs_latebf_indices, cs_bf_type_indices]),
        # np.zeros((CS_Data+1, CS_MaxDeathCauses+1, CS_AffecFract+1, CS_LateInitBF + 1, CS_NotBF + 1)).tolist(),
        
        CS_ImpactBFOnMortPostNatTag              : gb_shape('Impact of breastfeeding on post-neonatal mortality', '5D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_affect_fract_dim, cs_bf_type_dim, cs_age_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_affect_fract_indices, cs_bf_type_indices, cs_age_indices]),

        
        CS_RelRisksAnemiaTag                     : gb_shape('Relative risks of anemia', '4D float',
                                                            dim=[cs_data_dim, cs_cause_dim, cs_mat_anemia_dim, cs_affect_fract_dim],
                                                            indices=[cs_data_indices, cs_cause_indices, cs_mat_anemia_indices, cs_affect_fract_indices]),
        #  np.zeros((CS_Data + 1, CS_Mat_MaxDeathCauses + 1, CS_Mat_Anemic + 1, CS_AffecFract + 1)).tolist(),

        # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        # /////                                                                                                          /////
        # /////                                         Result Modvars                                                   /////
        # /////                                                                                                          /////
        # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        CS_PercDeathsAvtdSeqPTag                 : cs_cause_age_year_shape('Percentage of deaths averted by sequence P'),
        CS_MortalityRatesTag                     : gb_shape('Mortality rates', '3D float', dim=[cs_bounds_dim, cs_mort_dim, gb_year_dim], indices=[cs_bounds_indices, cs_mort_indices, gb_year_indices]),
        CS_DistPathDiarIncResultTag              : cs_diarr_paths_age_year_shape('Distribution of diarrhea incidence results'),
        
        # Neonatal, child sectors
        CS_ChDeathsTag                           : cs_age_year_shape('Child deaths'),
        CS_ChDeathsByYearTag                     : gb_shape('Child deaths by year', '3D float', dim=[cs_bounds_dim, cs_4t5Yr_dim, gb_year_dim], indices=[cs_bounds_indices, cs_4t5Yr_indices, gb_year_indices]),

        CS_ChDeathsPrevdTag                      : cs_age_year_shape('Child deaths prevented'),
        CS_ChDeathsByCauseTag                    : cs_cause_age_year_shape('Child deaths by cause'),
        CS_DeathsByCauseByBirthCohortTag         : cs_deaths_year_shape('Deaths by cause by birth cohort'),
        CS_ChDeathsPrevdByCauseTag               : cs_cause_age_year_shape('Child deaths prevented by cause'),
        CS_ChDeathsPrevdByIntTag                 : cs_interv_age_year_shape('Child deaths prevented by intervention'),
        CS_ChDeathsPrevByInterByCauseTag         : cs_interv_cause_age_year_shape('Child deaths prevented by intervention by cause'),
        CS_DetVaccCovResultsTag                  : gb_shape('Det vacc coverage results', '5D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_vacc_dose_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices,  cs_age_indices, gb_year_indices]),
        # gb_shape('', 'dict', 
        #                                                     innerShape={
        #                                                         'value' : ),
        #                                                         'retro' : gb_shape('retro', '5D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_vacc_dose_dim, cs_age_dim, cs_retro_years_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices, cs_age_indices, cs_retro_years_indices])
        #                                                     }),

        CS_DAvtdByVaccByCohortTag                : gb_shape('Deaths varted by vaccine by cohort', '4D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices,  cs_age_indices, gb_year_indices]),
        # gb_shape('', 'dict', 
        #                                                     innerShape={
        #                                                         'value' : ,
        #                                                         'retro' : gb_shape('retro', '4D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_age_dim, cs_retro_years_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_vacc_dose_indices, cs_age_indices, cs_retro_years_indices])
        #                                                     }),

        CS_DAvtdByVaccByCohortByCauseTag	     : gb_shape('value', '5D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_cause_dim, cs_age_dim, gb_year_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_cause_indices,  cs_age_indices, gb_year_indices]),
        # gb_shape('Deaths averated by vaccine by cohort by cause', 'dict', 
        #                                                     innerShape={
        #                                                         'value' : ,
        #                                                         'retro' : gb_shape('retro', '5D float', dim=[cs_bounds_dim, cs_last_vacc_dim, cs_cause_dim, cs_age_dim, cs_retro_years_dim], indices=[cs_bounds_indices, cs_last_vacc_indices, cs_cause_indices, cs_age_indices, cs_retro_years_indices])
        #                                                     }),

        # Maternal sector
        CS_MatDeathsTag                          : cs_year_shape('Maternal deaths'),
        CS_MatDeathsPrevTag                      : cs_year_shape('Maternal deaths prevented'),
        CS_MatDeathsByCauseTag                   : cs_mat_death_causes_by_year_shape('Maternal deaths by cause'),
        CS_MatDeathsPrevByCauseTag               : cs_mat_death_causes_by_year_shape('Maternal deaths prevented by cause'),
        CS_MatDeathsPrevByInterTag               : cs_interv_year_shape('Maternal deaths prevented by intervention'),
        CS_MatDeathsPrevByInterByCauseTag	     : cs_mat_deaths_prev_by_inter_shape('Maternal deaths prevented by intervention by cause'),
        CS_NumAbortionsTag	                     : cs_num_abortions_shape('Number of abortions'),

        # Stillbirth sector
        CS_SBResultsTag                          : cs_year_shape('Stillbirth results'),
        CS_SBPrevResultsTag                      : cs_year_shape('Stillbirth prevented results'),
        CS_SBByCauseResultsTag                   : cs_sbcauses_year('Stillbirth by cause results'),
        CS_SBPrevByCauseResultsTag               : cs_sbcauses_year('Stillbirth prevented by cause results'),
        CS_SBPrevByInterResultsTag               : cs_interv_year_shape('Stillbirth prevented by intervention results'),
        CS_SBPrevByInterByCauseTag               : cs_interv_sbcauses_year('Stillbirth prevented by intervention by cause results'),

        # Mortality rates sector
        CS_SBRateResultTag                       : cs_year_shape('Stillbirth rate results'),
        CS_MatMortRatioResultTag                 : cs_year_shape('Maternal mortality ratio results'),
        CS_MatMortRateResultTag                  : cs_year_shape('Maternal mortality rate results'),
        CS_MortalityRateResultTag                : gb_shape('Mortality rates result', '3D float', 
                                                            dim=[cs_bounds_dim, cs_mort_dim, gb_year_dim], 
                                                            indices=[cs_bounds_indices, cs_mort_indices, gb_year_indices]),
        CS_RedMortIntervResultTag                : cs_red_mort_interv_shape('Reductions in mortality by intervention results'),
        CS_RedMortIntervSortedIDsTag             : cs_interv_shape('Reductions in mortality by intervention sorted IDs'),

        # Nutrition sector
        CS_ShutDownBOAttriByIVTag                : cs_bool_shape('Shut down BO attributes by intervention'),
        CS_SubOpBOAvertByInterTag	             : cs_subop_bo_avert_by_inter_shape('SubOpBOAvertByInter'),
        CS_MAM_PINTag                            : cs_age_shape('MAM PIN'),
        CS_SAM_PINTag                            : cs_age_year_shape('SAM PIN'),
        CS_PInVarHtForAgeStuntTag	             : cs_std_age_shape('Percentage in variable height for age stunting'),
        CS_PInVarWtForHtWasteTag                 : cs_std_age_shape('Percentage in variable weight for height wasting'),
        CS_PInVarWtForHtWasteCFTag               : cs_std_age_shape('Percentage in variable weight for height wasting CF'),
        CS_NumStuntedTag                         : cs_age_year_shape('Number of stunted children'),
        CS_NumStuntedAvertedTag	                 : cs_age_year_shape('Number of stunted children averted'),
        CS_NumStuntedAvertedByInterTag	         : cs_interv_age_year_shape('Number of stunted children averted by intervention'),
        CS_PStuntRedAccountByIntBaseYrTag        : cs_interv_age_year_shape('Percentage stunt reduction accounted by intervention base year'),
        CS_AdditionalYrsSchoolingGainedTag	     : cs_year_shape('Additional years of schooling gained'),
        CS_PercIncEarnStuntDeclineTag	         : cs_year_shape('Percentage increase in earnings due to stunt reduction'),
        CS_ImpLifeEarnDueToStuntReducTag         : cs_discount_shape('Impact on lifetime earnings due to stunt reduction'),
        CS_LifetimeEarningsObtainedTag	         : cs_discount_shape('Lifetime earnings obtained'),
        CS_BFPrevResultTag                       : cs_bf_prev_shape('Breastfeeding prevalence results'),
        CS_BirthTermResultTag	                 : cs_birth_term_shape('Birth term results'),
        CS_PercAnemicWomenTag                    : cs_anemia_by_year_shape('Percentage of anemic women'),
        CS_NumAnemicWomenTag                     : cs_anemia_by_year_shape('Number of anemic women'),
        CS_NumAnemiaCasesPrevTag                 : cs_anemia_by_year_shape('Number of anemia cases prevented'),
        CS_PrevFoodInsecurityTag                 : cs_year_shape('Prevalence of food insecurity'),

        # Incidence and etiology sector
        CS_DiarrIncResultTag                     : cs_age_year_shape('Diarrhea incidence results'),
        CS_SevereDiarrIncResultTag               : cs_age_year_shape('Severe diarrhea incidence results'),
        CS_PneumIncResultTag                     : cs_age_year_shape('Pneumonia incidence results'),
        CS_MeninIncResultTag	                 : cs_age_year_shape('Meningitis incidence results'),
        CS_NumDiarrCasesTag                      : cs_age_year_shape('Number of diarrhea cases'),
        CS_NumSevereDiarrCasesTag                : cs_age_year_shape('Number of severe diarrhea cases'),
        CS_NumPneumCasesTag                      : cs_age_year_shape('Number of pneumonia cases'),
        CS_NumMeninCasesTag                      : cs_age_year_shape('Number of meningitis cases'),
        CS_DiarrCasesAvertByInterTag             : cs_bfpromotion_age_year_shape('Diarrhea cases averted by intervention'),
        CS_SevereDiarrCasesAvertByInterTag       : cs_bfpromotion_age_year_shape('Severe diarrhea cases averted by intervention'),
        CS_PneumCasesAvertByInterTag             : cs_pneum_age_year_shape('Pneumonia cases averted by intervention'),
        CS_MeninCasesAvertByInterTag             : cs_menin_age_year_shape('Meningitis cases averted by intervention'),
        CS_PercDiarrDeathsByPathTag	             : cs_diarr_paths_age_year_shape('Percentage of diarrhea deaths by path'),
        CS_PercPneumDeathsByPathTag              : cs_pneum_path_age_year_shape('Percentage of pneumonia deaths by path'),
        CS_PercMeninDeathsByPathTag              : cs_menin_path_age_year_shape('Percentage of meningitis deaths by path'),
        CS_DiarrDeathsByPathTag                  : cs_diarr_paths_age_year_shape('Diarrhea deaths by path'),
        CS_PneumDeathsByPathTag                  : cs_pneum_path_age_year_shape('Pneumonia deaths by path'),
        CS_MeninDeathsByPathTag                  : cs_menin_path_age_year_shape('Meningitis deaths by path'),
        CS_DiarrDeathsByPathBy1yrAgeGrpsTag      : cs_diarr_paths_agegroup_year_shape('Diarrhea deaths by path by 1 year age groups'),
        CS_PneumDeathsByPathBy1yrAgeGrpsTag      : cs_pneum_path_agegroup_year_shape('Pneumonia deaths by path by 1 year age groups'),
        CS_MeninDeathsByPathBy1yrAgeGrpsTag      : cs_menin_path_agegroup_year_shape('Meningitis deaths by path by 1 year age groups'),
        CS_PercDiarrCasesByPathTag	             : cs_diarr_paths_age_year_shape('Percentage of diarrhea cases by path'),
        CS_PercSevDiarrCasesByPathTag	         : cs_diarr_paths_age_year_shape('Percentage of severe diarrhea cases by path'),
        CS_PercPneumCasesByPathTag               : cs_pneum_path_age_year_shape('Percentage of pneumonia cases by path'),
        CS_PercMeninCasesByPathTag               : cs_menin_path_age_year_shape('Percentage of meningitis cases by path'),
        CS_DiarrCasesByPathTag                   : cs_diarr_paths_age_year_shape('Diarrhea cases by path'),
        CS_SevereDiarrCasesByPathTag             : cs_diarr_paths_age_year_shape('Severe diarrhea cases by path'),
        CS_PneumCasesByPathTag                   : cs_pneum_path_age_year_shape('Pneumonia cases by path'),
        CS_MeninCasesByPathTag                   : cs_menin_path_age_year_shape('Meningitis cases by path'),
        CS_DiarrCasesByPathBy1YrAgeGrpsTag       : cs_diarr_paths_agegroup_year_shape('Diarrhea cases by path by 1 year age groups'),
        CS_SevereDiarrCasesByPathBy1YrAgeGrpsTag : cs_diarr_paths_agegroup_year_shape('Severe diarrhea cases by path by 1 year age groups'),
        CS_PneumCasesByPathBy1YrAgeGrpsTag       : cs_pneum_path_agegroup_year_shape('Pneumonia cases by path by 1 year age groups'),
        CS_MeninCasesByPathBy1YrAgeGrpsTag       : cs_menin_path_agegroup_year_shape('Meningitis cases by path by 1 year age groups'),
        CS_DiarrCasesByBirthCohTag               : cs_year_shape('Diarrhea cases by birth cohort'),
        CS_PneumCasesByBirthCohTag               : cs_year_shape('Pneumonia cases by birth cohort'),
        CS_MeninCasesByBirthCohTag               : cs_year_shape('Meningitis cases by birth cohort'),
        CS_SevDiarrCasesByBirthCohTag            : cs_year_shape('Severe diarrhea cases by birth cohort'),
        CS_DiarCasesAvtdByYr1stDoseVaccTag       : cs_vacc_age_year_shape('Diarrhea cases averted by year 1st dose vaccine'),
        CS_PneumCasesAvtdByYr1stDoseVaccTag      : cs_vacc_age_year_shape('Pneumonia cases averted by year 1st dose vaccine'),
        CS_MeninCasesAvtdByYr1stDoseVaccTag      : cs_vacc_age_year_shape('Meningitis cases averted by year 1st dose vaccine'),
        CS_SevDiarrCasesAvtdByYr1stDoseVaccTag   : cs_vacc_age_year_shape('Severe diarrhea cases averted by year 1st dose vaccine'),
        CS_DiarrDeathsByPathBirthCohTag          : cs_diarr_paths_year_shape('Diarrhea deaths by path by birth cohort'),
        CS_PneumDeathsByPathBirthCohTag          : cs_pneum_path_year_shape('Pneumonia deaths by path by birth cohort'),
        CS_MeninDeathsByPathBirthCohTag          : cs_menin_path_year_shape('Meningitis deaths by path by birth cohort'),
        CS_DiarrCasesByPathBirthCohTag           : cs_diarr_paths_year_shape('Diarrhea cases by path by birth cohort'),
        CS_PneumCasesByPathBirthCohTag           : cs_pneum_path_year_shape('Pneumonia cases by path by birth cohort'),
        CS_MeninCasesByPathBirthCohTag           : cs_menin_path_year_shape('Meningitis cases by path by birth cohort'),
        CS_SevDiarrCasesByPathBirthCohTag        : cs_diarr_paths_year_shape('Severe diarrhea cases by path by birth cohort'),
        CS_DiarrDeathsByPathBirthCohVCTag        : cs_diarr_paths_year_shape('Diarrhea deaths by path by birth cohort VC'),
        CS_PneumDeathsByPathBirthCohVCTag        : cs_pneum_path_year_shape('Pneumonia deaths by path by birth cohort VC'),
        CS_MeninDeathsByPathBirthCohVCTag        : cs_menin_path_year_shape('Meningitis deaths by path by birth cohort VC'),
        CS_DiarrCasesByPathBirthCohVCTag         : cs_diarr_paths_year_shape('Diarrhea cases by path by birth cohort VC'),
        CS_PneumCasesByPathBirthCohVCTag         : cs_pneum_path_year_shape('Pneumonia cases by path by birth cohort VC'),
        CS_MeninCasesByPathBirthCohVCTag         : cs_menin_path_year_shape('Menin cases by path by birth cohort VC'),
        CS_PersonsVaccByIntervTag                : cs_vacc_dose_year_shape('Persons vaccinated by intervention'),
        CS_PersonsVaccByIntervBy1YrAgeGrpsTag    : cs_vacc_dose_agegroup_year_shape('Persons vaccinated by intervention by 1 year age groups'),

        CS_DiscountedCostsTag                    : cs_year_shape('Discounted costs'),
        CS_DiscountedSocialBenefitsTag           : cs_year_shape('Discounted social benefits'),
        CS_DiscountedEconBenefitsTag             : cs_year_shape('Discounted economic benefits'),
        CS_IncLifetimeEarningsDueToStuntTag      : cs_year_shape('Increase in lifetime earnings due to stunt reduction'),
        
        CS_TotalDiscountedCostsTag               : cs_int_shape('Total discounted costs'),
        CS_TotalDiscountedSocialBenefitsTag      : cs_int_shape('Total discounted social benefits'),
        CS_TotalDiscountedEconBenefitsTag        : cs_int_shape('Total discounted economic benefits'),
        CS_LifetimeEarningsIncTag                : cs_int_shape('Lifetime earnings increase due to stunt reduction'),
        CS_BCRTag                                : cs_int_shape('Benefit-cost ratio'),
}



TB_FirstHist_Year = 2000 # First year WHO data
TB_FinalHist_Year = 2024 # Final year WHO data, update when TB global report is updated
TB_FinalHist_Idx = TB_FinalHist_Year-TB_FirstHist_Year 
TB_FinalHist_Len = TB_FinalHist_Idx+1 

# TB_Base_Year  = 2021 #deafult to TB_FirstHist_Year
TB_Base_Year  = 2024 #default to TB_FirstHist_Year
TB_Min_First_Year = 2015
TB_Max_First_Year = 2026

# #Impact Models
# TB_StatisticalModel = "TB_StatisticalModel"
# TB_DynamicalModel   = "TB_DynamicalModel"

#Split by age
TB_Adults15p=0
TB_Chilren0to14=1

#Split by treatment history
TB_NewCases=0
TB_RetRelCases=1

#Split by pulmonary status
TB_PulmonaryTB=0
TB_ExtraPulmonaryTB=1

#Split by XDR status
TB_PreXDRorXDR=1 
TB_RR=2 # Rifampicine resitant, Pulmonary and Extra-Pulmonary TB

TB_noti_num=0 
TB_noti_perc=1

TB_HH_Contacts_A04  = 1
TB_HH_Contacts_A514 = 2
TB_HH_Contacts_A15p = 3





# HRG cohorts
# 1)Prisoners
# 2)Miners exposed to silica 
# 3)People with risk factors for TB seeking health care in settings with ? 0.1#  TB prevalence
# 4)Populations with structural risk factors for TB and limited access to health care
# 5)General population in settings with ? 0.5#  general prevalence
# 6)People with untreated fibrotic lesions on chest x-ray
# 7)Other high-risk groups
TB_HRG_Prisoners         = 0
TB_HRG_Miners            = 1
TB_HRG_SeekingCare       = 2
TB_HRG_StructuralRFs     = 3
TB_HRG_HighTBPrev        = 4
TB_HRG_UntreatedFibrosis = 5
# TB_HRG_Other             = 6
TB_HRG_Len               = 6

TB_HRG_List = [
    TB_HRG_Prisoners        ,
    TB_HRG_Miners           ,
    TB_HRG_SeekingCare      ,
    TB_HRG_StructuralRFs    ,
    TB_HRG_HighTBPrev       ,
    TB_HRG_UntreatedFibrosis,
]

#HRG demographic indexes
TG_HRG_Dem_TotalPop  = 0 
TG_HRG_Dem_Prop15p   = 1
TG_HRG_Dem_TBInfect  = 2
TB_HRG_Dem_TBDisease = 3
TB_HRG_Dem_RelRisk   = 4
TB_HRG_Dem_PTB       = 5
TG_HRG_DemoGraphics_Len = 6


# Pulmonary TB
TB_PTB_HIVn_014_grp  = 0
TB_PTB_HIVn_15p_grp  = 1
TB_PTB_HIVp_09_grp   = 2
TB_PTB_HIVp_1014_grp = 3
TB_PTB_HIVp_15p_grp  = 4

# Extra-Pulmonary TB
TB_ETB_HIVn_014_grp  = 5
TB_ETB_HIVn_15p_grp  = 6
TB_ETB_HIVp_09_grp   = 7
TB_ETB_HIVp_1014_grp = 8
TB_ETB_HIVp_15p_grp  = 9

TB_Pulmonary_count = 10


TB_Pulmonary_list = [
    TB_PTB_HIVn_014_grp, 
    TB_PTB_HIVn_15p_grp, 
    TB_PTB_HIVp_09_grp, 
    TB_PTB_HIVp_1014_grp,
    TB_PTB_HIVp_15p_grp, 
    TB_ETB_HIVn_014_grp, 
    TB_ETB_HIVn_15p_grp, 
    TB_ETB_HIVp_09_grp, 
    TB_ETB_HIVp_1014_grp,
    TB_ETB_HIVp_15p_grp 
]


TB_ScreenedNum         = 0
TB_ScreenedNumReferred = 1
TB_ScreenedTP          = 2
TB_ScreenedFP          = 3
TB_ScreenedTrueCases   = 4
TB_ScreenedNNTS        = 5
TB_ScreenedTPFP        = 6
TB_ScreenedTPMissed    = 7
TB_ScreenOutputLen     = 8

TB_DiagnosisNum       = 0 
TB_DiagnosisConfirmed = 1
TB_DiagnosisTP        = 2
TB_DiagnosisFP        = 3
TB_DiagnosisTrueCases = 4
TB_DiagnosisNNTT      = 5
TB_DiagnosisTPFP      = 6
TB_DiagnosisTPMissed  = 7
TB_DiagnosisLen       = 8  

TB_ClinicallyAssessed = 0
TB_ReferredDx         = 1  
TB_DiagnosisTB        = 2  
TB_TreatmentInit      = 3
TB_PatientTargPopLen  = 4          


# ART cohorts
TB_ARTCohort_A09_NotSeriouslyIll   = 1
TB_ARTCohort_A1014_NotSeriouslyIll = 2
TB_ARTCohort_A15p_NotSeriouslyIll  = 3
TB_ARTCohort_A09_SeriouslyIll      = 4
TB_ARTCohort_A1014_SeriouslyIll    = 5
TB_ARTCohort_A15p_SeriouslyIll     = 6


# Clinical assessment algorithm used
TB_ClinicalAssessment = 1  # Clinical assessment: any symptom 
TB_ChestXRay          = 2  # Chest X-ray [ digital (dxr) vs conventional (cxr) ] [ screening test ]
TB_ComputerAidedDtct  = 3  # Computer-aided detection software for automated reading of digital chest radiographs [ screening test ]
TB_W3SS               = 4  # W3SS (3 symptom) single screening algorithm
TB_4SymptomScreening  = 5  # 4 symptom screening (no test) / History and physical examination and information on TB
TB_C_ReactiveProtein  = 6  # C-reactive protein with a cut-off of > 5 mg/L [ screening test ]
TB_ClinicalExtraPulm  = 7  # Clinical assessment: any extra-pulmonary TB symptom* 
TB_W4SS               = 8  # W4SS
TB_SingleSymptomScrn  = 9  # Single Symptom screening
TB_TuberculinSkinTest = 10 # Tuberculin Skin Test (TST) 
TB_AntigenSkinTest    = 11 # TB antigen-based skin tests for the diagnosis of TB infection (TBST)  *, 0% 
TB_IGRA               = 12 # Interferon-Gamma Release Assay (IGRA)*, 100%
TB_mWRDPrevGT10       = 13 # mWRD single screening algorithm for medical inpatients in settings with TB prevalence > 10%
TB_FirstlinePreferred = 14 # First-line regimen (preferred): TDF + 3TC (or FTC) + DTG
TB_FirstlineAltern    = 15 # First-line regimen (alternative): TDF + 3TC + EFV 400 mg
TB_SpecialCircTDF_EFV = 16 # Special circumstances: TDF + 3TC (or FTC) + EFV 600mg
TB_SpecialCircAZT     = 17 # Special circumstances:  AZT + 3TC+ EFV 600mg
TB_SpecialCircTDF_PI  = 18 # Special circumstances:  TDF + 3TC (or FTC)+ PI
TB_SpecialCircTDF_RAL = 19 # Special circumstances:  TDF + 3TC (or FTC)+  RAL
TB_SpecialCircTAF     = 20 # Special circumstances:  TAF + 3TC (or FTC)+ DTG
TB_SpecialCircABC     = 21 # Special circumstances:  ABC+ 3TC + DTG
TB_FirstlineABC_DTG   = 22 # First-line regimen (preferred):  ABC + 3TC + DTG
TB_FirstlineABC_LPV   = 23 # First-line regimen (alternative): ABC + 3TC + LPV/r
TB_FirstlineTAF_DTG   = 24 # First-line regimen (alternative): TAF + 3TC (or FTC) + DTG
TB_SpecialCircABC_EFV = 25 # Special circumstances:  ABC+ 3TC + EFV (or NVP)
TB_SpecialCircABC_RAL = 26 # Special circumstances: ABC + 3TC + RAL
TB_SpecialCircAZT_LPV = 27 # Special circumstances: AZT + 3TC + EFV(or NVP)
TB_SpecialCircAZT_EFV = 28 # Special circumstances: AZT + 3TC + LPV/r (or RAL)


#TB Screening methods
TB_ScrnAnySympCXRMeth = 0
TB_ScrnW3SSMeth       = 1
TB_ScrnW4SSCRPMeth    = 2
TB_ScrnClinicAssMeth  = 3


#TB notification case types
TB_NewCases                       = 0
TB_NewBacterConfirmCases          = 1 #Pulmonary, bacteriologically confirmed
TB_NewClinicDiagCases             = 2 #Pulmonary, clinically diagnosed,
TB_NewExtrapulmonaryCases         = 3 #Extrapulmonary,
TB_RelapseCases                   = 4 #  
TB_RelapseBacterConfirmCases      = 5 #Pulmonary, bacteriologically confirmed
TB_RelapseClinicDiagCases         = 6 #Pulmonary, clinically diagnosed,
TB_RelapseExtrapulmonaryCases     = 7 #Extrapulmonary,
TB_NewAndRelapsedCases            = 8 #
TB_PrevTreatExcludeRelapseCases   = 9 #Previously treated, excluding relapse
TB_PrevTreatIncludeRelapseCases   = 10 #Previously treated, including relapse
#Split by age*                    
TB_NewRelapse0_9Cases             = 11 #Children 0-9 years\, 
TB_NewRelapse10_14Cases           = 12 #Children 10-14 years\, new and relapse
TB_NewRelapse15pCases             = 13 #Adults 15+ years
#Split by HIV status*             
TB_HIVpCases                      = 14 #HIV+
TB_HIVp15pCases                   = 15 #HIV+ Adults 15+ years
TB_HIVp0_14Cases                  = 16 #HIV+ Children 0-14 years
TB_HIVpNeonateCases               = 17 #HIV+ Neonates
TB_NotOnARTCases                  = 18 #Not on ART
TB_OnARTCases                     = 19 #On ART
TB_OnART15pCases                  = 20 #On ART Adults 15+ years, failing this frst-line regimen 
TB_OnARRT0_14Cases                = 21 #On ART Children 0-14 years, failing this first-line regimen
TB_OnARTNeonateCases              = 22 #On ART Neonates, failing this first-line regimen
TB_HIVpExcludeART                 = 23 #Proportion HIVp in pop that excludes ART
TB_NotificationCaseCount = 24


TB_AbbottRealTimeMTB  = 0  # Abottv Real TIME MTB and MTB RIF/INH assays (Abbott Laboratories, Abbott Park, USA) - only on sputum sample
TB_BDMAXMDRTBAssay    = 1  # BD MAX MDR-TB assay (Becton, Dickinson and Company, Franklin Lakes, USA) - only on sputum sample
TB_BloodBoneMarrow    = 2  # blood or bone marrow
TB_CerebrospinalFluid = 3  # cerebrospinal fluid (CSF) and lumbar puncture
TB_CultureBactConfirm = 4  # Culture (liquid) - for bacteriological confirmation
TB_CultureDiagTB      = 5  # Culture (liquid) for diagnosis of TB
TB_DiagKidAspirates   = 6  # Diagnostic procedure (children)  - gastric aspirates [recommended for < 7 years]
TB_DiagKidSputum      = 7  # Diagnostic procedure (children)  - Induced sputum 
TB_DiagKidnasopharyn  = 8  # Diagnostic procedure (children)  - nasopharyngeal aspiration [recommended for < 7 years]
TB_DiagKidBronchos    = 9  # Diagnostic procedure (children) - Bronchoscopy (Bronchoalveolar lavage)
TB_DiagKidGasLavage   = 10 # Diagnostic procedure (children)  -gastric lavage [recommended for < 7 years]
TB_DiagKidStools      = 11 # Diagnostic procedure (children) - Stools sampling + Xpert MTB/RIF or Ultra)
TB_Disseminated       = 12 # Disseminated TB
TB_FineNeedleAspir    = 13 # Fine-needle aspiration or biopsy other than lymph node
TB_HainFlouroType     = 14 # Hain FluoroType MTBDR assay (Bruker/Hain Lifescience, Nehren, Germany) - only on sputum sample
TB_LymphNodeAspirate  = 15 # Lymph node aspirate
TB_LymphNodeBiopsy    = 16 # Lymph node biopsy
TB_MicroscopyConvent  = 17 # Microscopy conventional
TB_MicroscopyLED      = 18 # Microscopy LED
TB_ModCmplxAbbott     = 19 # Moderate complexity automated NAAT: Abbott RealTime MTB and MTB RIF/INH assays (Abbott Laboratories, Abbott Park, USA)
TB_ModCmplxBDMAXMDR   = 20 # Moderate complexity automated NAAT: BD MAX MDR-TB assay (Becton, Dickinson and Company, Franklin Lakes, USA)
TB_ModCmplxHainFlouro = 21 # Moderate complexity automated NAAT: Hain FluoroType MTBDR assay (Bruker/Hain Lifescience, Nehren, Germany)
TB_ModCmplxRocheCobas = 22 # Moderate complexity automated NAAT: Roche cobas MTB and MTB-RIF/INH assays (Hoffmann-La Roche, Basel, Switzerland)
TB_OtherTypes         = 23 # Other types of TB
TB_PericardialFluid   = 24 # Pericardial fluid
TB_PeritonealFluid    = 25 # Peritoneal fluid
TB_PlueralFluid       = 26 # Pleural fluid
TB_RocheCobasMTBAssay = 27 # Roche cobas MTB and MTB-RIF/INH assays (Hoffmann-La Roche, Basel, Switzerland) - only on sputum sample
TB_SerosalFluid       = 28 # Serosal fluid aspirate with serosal tissue biopsy
TB_SynovialFluid      = 29 # Synovial fluid
TB_MeningitisTesting  = 30 # TB meningitis testing
TB_TissueBiopsy       = 31 # TB tissue biopsy or aspirates 
TB_LAMPEikenChemical  = 32 # TB-LAMP; Eiken Chemical, Tokyo, Japan
TB_LAMPEikenSputum    = 33 # TB-LAMP; Eiken Chemical, Tokyo, Japan - only on sputum sample
TB_TruenatMTB_mWRD    = 34 # Truenat MTB  (mWRD)
TB_TruenatMTB_Sputum  = 35 # Truenat MTB  (mWRD) - only on sputum sample
TB_TruenatMTBp_mWRD   = 36 # Truenat MTB Plus  (mWRD)
TB_TruenatMTBp_Sputum = 37 # Truenat MTB Plus  (mWRD) - only on sputum sample
TB_Urine              = 38 # Urine (for tract TB detection only)
TB_XpertMTB_RIF       = 39 # Xpert MTB/RIF (mWRD)
TB_XpertUltra         = 40 # Xpert Ultra  (mWRD)
TB_CD4Count           = 41 # CD4 count
TB_LFLAM              = 42 # LF-LAM [ Biomarker-based lateral flow lipoarabinomannan assay (LF-LAM) test (Alere Determine TB LAM Ag, USA
TB_TruenatMTB_RIF     = 43 # Truenat MTB RIF D x  (mWRD)  
TB_ConcurrentPLHIV    = 44 # Concurrent testing in PLHIV: LC-aNAAT + LF-LAM	44
TB_ConcurrentChildren = 45 # Concurrent testing in children: LC-aNAAT on respiratory sample (i.e. sputum) + stool	45
TB_ConcurrentCLHIV    = 46 # Concurrent testing in CLHIV: LC-aNAAT on respiratory sample  + LC-aNAAT on stool + LF-LAM	46


#TB Diagnostic methods
TB_MethodOnlySputum        = 0 # mWRD [ all samples ] and Moderate complexity automated NAAT [ MC- aNAAT ] [ only on sputum samples ] [ Choice ]
TB_MethodDiagProcTBInKids  = 1 # Diagnostic procedure\, pulmonary TB evaluation in children
TB_MethodMicroscopyCulture = 2 # Microscropy (conventional vs LED) and culture
TB_MethodmWRDModComplex    = 3 # mWRD and Moderate complexity automated NAAT [ MC - aNAAT] [ choice ]*
TB_MethodNonSputumETB      = 4 # Non-sputum sample extraction for EPTB
TB_MethodmWRD_ETB          = 5 # mWRD and Moderate complexity automated NAAT (choice) with sample extraction for EPTB
TB_MethodLAM_mWRD          = 6 # LF-LAM followed by mWRD 
TB_MethodLAM_LAMP          = 7 # LF-LAM\, TB-LAMP\, CD4-count
TB_MethodCustom            = 8 # Custom method     



TB_TotalContacts      = 0
TB_DiseaseInfectCntct = 1
TB_InfectionCntct     = 2 
TB_DiseaseCntct       = 3
TB_PulmTBCntct        = 4
TB_ContactLen         = 5

TB_HH0_4  = 0 
TB_HH5_14 = 1
TB_HH15p  = 2
TB_HHAgeLen = 3
TB_HHList= [
    TB_HH0_4  , 
    TB_HH5_14 ,
    TB_HH15p
]

TB_ART0_9  = 0 
TB_ART10_14 = 1
TB_ART15p  = 2
TB_ARTAgeLen = 3
TB_ARTAgeList = [
    TB_ART0_9,
    TB_ART10_14,
    TB_ART15p
]

TB_ARTNotServere    = 0
TB_ARTSevereIllness = 1
TB_ARTSevereLen     = 2


TB_NumEligibleScreen = 0
TB_NumScreened       = 1
TB_NumReferredDiag   = 2 
TB_NumDiagPulmTB     = 3
TB_NumInitTreatment  = 4
TB_NumEligibleEval   = 5
TB_NumTestedInfect   = 6
TB_NumConfirmInfect  = 7
TB_NumInitTPT        = 8
TB_TargetPopLen      = 9


TB_ART_LHIV_Num = 0
TB_ART_PercWithoutSerIll = 1
TB_ART_LHIV_Len = 2


TB_ART_WithTBInfection = 0
TB_ART_WithTBDisease   = 1
TB_ART_PercPTB         = 2
TB_ART_Charac_len      = 3

TB_SEN = 0
TB_MDR = 1
TB_ResistanceLen = 2

TB_ResistAge0_14 = 0
TB_ResistAge15p  = 1
TB_ResistAgeLen  = 2

TB_NewCases=0
TB_RetreatCases = 1
TB_NewRetreatLen = 2




TB_Rif_Sen = 0
TB_INH_Sen = 1
TB_INH_Res = 2
TB_Rif_Res = 3
TB_FQ_Sen  = 4
TB_FQ_Res  = 5
TB_XDR     = 6
TB_ResistProfileLen = 7


TB_Cov_RR_DST_PTB          =  0
TB_Cov_RR_DST_ETB          =  1
TB_Cov_INH_RR_Sens_DST_PTB =  2
TB_Cov_INH_RR_Sens_DST_ETB =  3
TB_Cov_FQ_RR_Res_DST_PTB   =  4
TB_Cov_FQ_RR_Res_DST_ETB   =  5
TB_Cov_BDQ_RR_Res_DST_PTB  =  6
TB_Cov_BDQ_RR_Res_DST_ETB  =  7
TB_Cov_BDQ_FQ_Res_DST_PTB  =  8
TB_Cov_BDQ_FQ_Res_DST_ETB  =  9
TB_Cov_LZD_RR_Res_DST_PTB  = 10
TB_Cov_LZD_RR_Res_DST_ETB  = 11
TB_Cov_LZD_FQ_Res_DST_PTB  = 12
TB_Cov_LZD_FQ_Res_DST_ETB  = 13
TB_Cov_EMC_XDR_DST_XDR     = 14
TB_Cov_EMC_RR_Res_DST_TB   = 15
TB_Cov_ETO_XDR_DST_XDR     = 16
TB_Cov_ETO_RR_Res_DST_TB   = 17
TB_Cov_PZA_XDR_DST_XDR     = 18
TB_Cov_PZA_RR_Res_DST_TB   = 19
TB_CovXDR_BDQ_LZD_MOX      = 20
TB_CoverageLen = 21

TB_NumTestPTB_RIF_INH_MDR    = 0
TB_NumTestPTB_INHMDR_RIFSens = 1
TB_NumTestPTB_FQ_RIF_MDR     = 2
TB_NumTestPTB_BDQ_RIF_MDR    = 3
TB_NumTestPTB_BDQ_FQ_MDR     = 4
TB_NumTestPTB_LZD_RIF_MDR    = 5
TB_NumTestPTB_LZD_FQ_MDR     = 6
TB_NumTestETB_RIF_INH_MDR    = 7
TB_NumTestETB_INHMDR_RIFSens = 8
TB_NumTestETB_FQ_RIF_MDR     = 9
TB_NumTestETB_BDQ_RIF_MDR    = 10
TB_NumTestETB_BDQ_FQ_MDR     = 11
TB_NumTestETB_LZD_RIF_MDR    = 12
TB_NumTestETB_LZD_FQ_MDR     = 13
TB_NumTextXDR_Amikacin       = 14
TB_NumTextXDR_Ethionamide    = 15
TB_NumTextXDR_Pyrazinamide   = 16
TB_NumTextXDR_BDQ_LZD_MOX    = 17

TB_NumTestLen = 18

TB_NumConfRIF_INHSen      = 0
TB_NumConfRIFSen_INHMDR   = 1			
TB_NumConfRIFMDR_FQSen    = 2
TB_NumConfRIFMDR_FQMDR    = 3
TB_NumConfAtLeastOne      = 4
TB_NumConfMeningitisPeri  = 5
TB_NumConfResistLen       = 6 

TB_Child = 0
TB_Adult = 1
TB_ChildAdultLen = 2

TB_PTB = 0
TB_ETB = 1
TB_PulLen = 2

#HIV TB CoInfection
TB_HIVCoInf             = 0 # Number PLHIV with TB 
TB_HIVCoInfAdult        = 1 # Number PLHIV with TB, adults';
TB_HIVCoInfChild        = 2 # Number PLHIV with TB, children';
TB_HIVCoInfNeonates     = 3 # Number PLHIV with TB, neonates';
TB_HIVCoInfFailAdult    = 4 # Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, adults';
TB_HIVCoInfFailChild    = 5 # Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, children';
TB_HIVCoInfFailNeonates = 6 # Number failing this first-line HIV regimen: TDF + 3TC (or FTC) + DTG, neonates';
TB_HIVCoInfLen = 7


# Scenarios 
TB_AvgProbDetect      = 0 
TB_AvgProbReceivPrvnt = 1
TB_TreatmentSuccess   = 2
TB_ScenarioLen = 3
   
# 
TB_Counterfactual = 0 
TB_Scaleup        = 1
TB_Difference     = 2
TB_ImpactResultLen = 3

TB_ResLowerIdx   = 0
TB_ResMidIdx   = 1 
TB_ResUpperIdx = 2
TB_ResSdIdx    = 3 
TB_ResultBounds = 4

TB_Scrn_g_sym_cgh2w = 'g_sym_cgh2w'
TB_Scrn_g_sym_any = 'g_sym_any'
TB_Scrn_g_mwrd_sn = 'g_mwrd_sn'
TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_pos = 'g_sym_cgh2w_cxr_tb_sequential_pos'
TB_Scrn_g_sym_cgh2w_cxr_tb_sequential_neg = 'g_sym_cgh2w_cxr_tb_sequential_neg'
TB_Scrn_g_sym_cgh2w_cxr_tb_parallel = 'g_sym_cgh2w_cxr_tb_parallel'
TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_pos = 'g_sym_cgh2w_cxr_cad_sequential_pos'
TB_Scrn_g_sym_cgh2w_cxr_cad_sequential_neg = 'g_sym_cgh2w_cxr_cad_sequential_neg'
TB_Scrn_g_sym_cgh2w_cxr_cad_parallel = 'g_sym_cgh2w_cxr_cad_parallel'
TB_Scrn_g_sym_any_cxr_tb_sequential_pos = 'g_sym_any_cxr_tb_sequential_pos'
TB_Scrn_g_sym_any_cxr_tb_sequential_neg = 'g_sym_any_cxr_tb_sequential_neg'
TB_Scrn_g_sym_any_cxr_tb_parallel = 'g_sym_any_cxr_tb_parallel'
TB_Scrn_g_sym_any_cxr_cad_sequential_pos = 'g_sym_any_cxr_cad_sequential_pos'
TB_Scrn_g_sym_any_cxr_cad_sequential_neg = 'g_sym_any_cxr_cad_sequential_neg'
TB_Scrn_g_sym_any_cxr_cad_parallel = 'g_sym_any_cxr_cad_parallel'
TB_Scrn_h_clhiv_sym = 'h_clhiv_sym'
TB_Scrn_h_hiv_mwrd_sn = 'h_hiv_mwrd_sn'
TB_Scrn_h_hiv_w4ss = 'h_hiv_w4ss'
TB_Scrn_h_hiv_crp = 'h_hiv_crp'
TB_Scrn_h_hiv_cxr_abn = 'h_hiv_cxr_abn'
TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_pos = 'h_hiv_w4ss_hiv_crp_sequential_pos'
TB_Scrn_h_hiv_w4ss_hiv_crp_sequential_neg = 'h_hiv_w4ss_hiv_crp_sequential_neg'
TB_Scrn_h_hiv_w4ss_hiv_crp_parallel = 'h_hiv_w4ss_hiv_crp_parallel'
TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_pos = 'h_hiv_w4ss_hiv_cxr_abn_sequential_pos'
TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_sequential_neg = 'h_hiv_w4ss_hiv_cxr_abn_sequential_neg'
TB_Scrn_h_hiv_w4ss_hiv_cxr_abn_parallel = 'h_hiv_w4ss_hiv_cxr_abn_parallel'
TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_pos = 'h_hiv_w4ss_cxr_cad_sequential_pos'
TB_Scrn_h_hiv_w4ss_cxr_cad_sequential_neg = 'h_hiv_w4ss_cxr_cad_sequential_neg'
TB_Scrn_h_hiv_w4ss_cxr_cad_parallel = 'h_hiv_w4ss_cxr_cad_parallel'
TB_Scrn_c_cc_sym = 'c_cc_sym'
TB_Scrn_c_cc_cxr_tb = 'c_cc_cxr_tb'
TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_pos = 'c_hiv_w4ss_cc_cxr_tb_sequential_pos'
TB_Scrn_c_hiv_w4ss_cc_cxr_tb_sequential_neg = 'c_hiv_w4ss_cc_cxr_tb_sequential_neg'
TB_Scrn_c_hiv_w4ss_cc_cxr_tb_parallel = 'c_hiv_w4ss_cc_cxr_tb_parallel'
TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_pos = 'c_hiv_w4ss_cxr_cad_sequential_pos'
TB_Scrn_c_hiv_w4ss_cxr_cad_sequential_neg = 'c_hiv_w4ss_cxr_cad_sequential_neg'
TB_Scrn_c_hiv_w4ss_cxr_cad_parallel = 'c_hiv_w4ss_cxr_cad_parallel'
TB_Scrn_1Step_custom = '1step_custom'
TB_Scrn_2Step_custom = '2step_custom'
TB_Scrn_custom       = 'custom_'



TB_Cust_Scrn_cxr_abn     = 'cxr_abn'
TB_Cust_Scrn_cxr_tb      = 'cxr_tb'
TB_Cust_Scrn_cxr_cad     = 'cxr_cad'
TB_Cust_Scrn_sym_cgh2w   = 'sym_cgh2w'
TB_Cust_Scrn_sym_cghany  = 'sym_cghany'
TB_Cust_Scrn_sym_any     = 'sym_any'
TB_Cust_Scrn_mwrd_sn     = 'mwrd_sn'
TB_Cust_Scrn_cxr_second  = 'cxr_second'
TB_Cust_Scrn_hiv_w4ss    = 'hiv_w4ss'
TB_Cust_Scrn_hiv_cxr_abn = 'hiv_cxr_abn'
TB_Cust_Scrn_hiv_crp     = 'hiv_crp'
TB_Cust_Scrn_hiv_mwrd_sn = 'hiv_mwrd_sn'
TB_Cust_Scrn_clhiv_sym   = 'clhiv_sym'
TB_Cust_Scrn_cc_sym      = 'cc_sym'
TB_Cust_Scrn_cc_cxr_tb   = 'cc_cxr_tb'
TB_Cust_Scrn_ssm_sz      = 'ssm_sz'
TB_Cust_Scrn_xpert       = 'xpert'


TB_Cust_SensSpec = {
    TB_Cust_Scrn_cxr_abn     : (0.94, 0.89),
    TB_Cust_Scrn_cxr_tb      : (0.85, 0.96),
    TB_Cust_Scrn_cxr_cad     : (0.94, 0.89),
    TB_Cust_Scrn_sym_cgh2w   : (0.42, 0.94),
    TB_Cust_Scrn_sym_cghany  : (0.51, 0.88),
    TB_Cust_Scrn_sym_any     : (0.71, 0.64),
    TB_Cust_Scrn_mwrd_sn     : (0.69, 0.99),
    TB_Cust_Scrn_cxr_second  : (0.90, 0.41),
    TB_Cust_Scrn_hiv_w4ss    : (0.83, 0.38),
    TB_Cust_Scrn_hiv_cxr_abn : (0.93, 0.20),
    TB_Cust_Scrn_hiv_crp     : (0.90, 0.50),
    TB_Cust_Scrn_hiv_mwrd_sn : (0.69, 0.98),
    TB_Cust_Scrn_clhiv_sym   : (0.61, 0.94),
    TB_Cust_Scrn_cc_sym      : (0.89, 0.69),
    TB_Cust_Scrn_cc_cxr_tb   : (0.84, 0.91),
    TB_Cust_Scrn_ssm_sz      : (0.61, 0.98),
    TB_Cust_Scrn_xpert       : (0.85, 0.98),
}


TB_Sensitivity_index = 0
TB_Specificity_index = 1


TB_Fort_Final_Year = 2050 #  model values are good up to 2030


TB_DynAge0_4   = 0 
TB_DynAge5_9   = 1 
TB_DynAge10_14 = 2 
TB_DynAge15_64 = 3 
TB_DynAge65p   = 4
TB_DynAgeLen   = 5

TB_DynHIVn       = 0
TB_DynHIVpNoART  = 1
TB_DynHIVpOnART  = 2
TB_DynHIVARTLen  = 3

# TB_DynHIVn    = 0 Already declared
TB_DynHIVp    = 1
TB_DynHIVLen  = 2

TB_DynRelapseTreatment = 0 
TB_DynRelapseSelfCure  = 1
TB_DynRelapseGt2Yrs    = 2
TB_DynRelapseLen       = 3

TB_DynPublicSector  = 0 
TB_DynPrivateSector = 1
TB_DynSectorLen     = 2

TB_DynPreExposure  = 0
TB_DynPostExposire = 1
TB_DynExposureLen  = 2


TB_general_public_screenings = [
TB_Cust_Scrn_cxr_abn,	  
TB_Cust_Scrn_cxr_tb,	  
TB_Cust_Scrn_cxr_cad,	  
TB_Cust_Scrn_sym_cgh2w,
TB_Cust_Scrn_sym_cghany,
TB_Cust_Scrn_sym_any,	  
TB_Cust_Scrn_mwrd_sn,	  
TB_Cust_Scrn_cxr_second,
TB_Cust_Scrn_ssm_sz, 
TB_Cust_Scrn_xpert]

TB_PLHIV_screenings = [
TB_Cust_Scrn_hiv_w4ss,	
TB_Cust_Scrn_hiv_cxr_abn,
TB_Cust_Scrn_hiv_crp,	   
TB_Cust_Scrn_hiv_mwrd_sn,
TB_Cust_Scrn_ssm_sz, 
TB_Cust_Scrn_xpert]

TB_CLHIV_screenings = [
TB_Cust_Scrn_clhiv_sym,       
TB_Cust_Scrn_ssm_sz, 
TB_Cust_Scrn_xpert]

TB_child_contact_screenings = [
TB_Cust_Scrn_cc_sym,
TB_Cust_Scrn_cc_cxr_tb,         
TB_Cust_Scrn_ssm_sz, 
TB_Cust_Scrn_xpert]

# Pulmonary TB	
TB_Cust_patient_menu =[ 
 TB_general_public_screenings, #  TB_PTB_HIVn_014_grp, 
 TB_general_public_screenings, #  TB_PTB_HIVn_15p_grp, 
 TB_CLHIV_screenings, #  TB_PTB_HIVp_09_grp, 
 TB_CLHIV_screenings, #  TB_PTB_HIVp_1014_grp, 
 TB_PLHIV_screenings, #  TB_PTB_HIVp_15p_grp, 
# Extra-Pulmonary TB	
 TB_general_public_screenings, # TB_ETB_HIVn_014_grp 
 TB_general_public_screenings, # TB_ETB_HIVn_15p_grp	
 TB_CLHIV_screenings, # TB_ETB_HIVp_09_grp	
 TB_CLHIV_screenings, # TB_ETB_HIVp_1014_grp
 TB_PLHIV_screenings] # TB_ETB_HIVp_15p_grp	

TB_Cust_patient_default =[ 
 TB_general_public_screenings[0], #  TB_general_public_screenings, 
 TB_general_public_screenings[0], #  TB_general_public_screenings, 
 TB_CLHIV_screenings[0], #  TB_CLHIV_screenings, 
 TB_CLHIV_screenings[0], #  TB_CLHIV_screenings, 
 TB_PLHIV_screenings[0], #  TB_PLHIV_screenings, 
# Extra-Pulmonary TB	
 TB_general_public_screenings[0], # TB_ETB_HIVn_014_grp 
 TB_general_public_screenings[0], # TB_ETB_HIVn_15p_grp	
 TB_CLHIV_screenings[0], # TB_ETB_HIVp_09_grp	
 TB_CLHIV_screenings[0], # TB_ETB_HIVp_1014_grp
 TB_PLHIV_screenings[0]] # TB_ETB_HIVp_15p_grp	

# ART cohorts	
TB_Cust_ART_NotSerous_menu =[
	TB_CLHIV_screenings, # Age 0-9 
	TB_CLHIV_screenings, # Age 10-14 
	TB_PLHIV_screenings  # Age 15+ 
]
TB_Cust_ART_Serious_menu = [
    TB_CLHIV_screenings, # Age 0-9 
    TB_CLHIV_screenings, # Age 10-14 
    TB_PLHIV_screenings  # Age 15+ 
]
TB_Cust_ART_menu =[TB_Cust_ART_NotSerous_menu, TB_Cust_ART_Serious_menu]


TB_Cust_ART_NotSerous_default =[
	TB_CLHIV_screenings[0], # Age 0-9 
	TB_CLHIV_screenings[0], # Age 10-14 
	TB_PLHIV_screenings[0] #  Age 15+ 
 ]
TB_Cust_ART_Serious_default = [
    TB_CLHIV_screenings[0], #  Age 0-9 
    TB_CLHIV_screenings[0], #  Age 10-14 
    TB_PLHIV_screenings[0]  #  Age 15+ 
]
TB_Cust_ART_default =[TB_Cust_ART_NotSerous_default, TB_Cust_ART_Serious_default]
#HH contacts	
TB_Cust_HH_menu = [
    TB_child_contact_screenings, # TB_HH0_4	
    TB_child_contact_screenings, # TB_HH5_14	
    TB_general_public_screenings, # TB_HH15p	
]
TB_Cust_HH_default = [
    TB_child_contact_screenings[0], # TB_HH0_4	
    TB_child_contact_screenings[0], # TB_HH5_14	
    TB_general_public_screenings[0], # TB_HH15p	
]
#High risk groups	
TB_Cust_HRG_menu = [
    TB_general_public_screenings, # TB_HRG_Prisoners	    
    TB_general_public_screenings, # TB_HRG_Miners	        
    TB_general_public_screenings, # TB_HRG_SeekingCare	    
    TB_general_public_screenings, # TB_HRG_StructuralRFs	
    TB_general_public_screenings, # TB_HRG_HighTBPrev	    
    TB_general_public_screenings, # TB_HRG_UntreatedFibrosis
]

TB_Cust_HRG_default = [
    TB_general_public_screenings[0], # TB_HRG_Prisoners	    
    TB_general_public_screenings[0], # TB_HRG_Miners	        
    TB_general_public_screenings[0], # TB_HRG_SeekingCare	    
    TB_general_public_screenings[0], # TB_HRG_StructuralRFs	
    TB_general_public_screenings[0], # TB_HRG_HighTBPrev	    
    TB_general_public_screenings[0], # TB_HRG_UntreatedFibrosis
]

TB_NOTIF_DB_KEYS = ['new_labconf', 'new_clindx', 'new_ep', 
                     'ret_rel_labconf','ret_rel_clindx', 'ret_rel_ep',   # Retreatment and relapse patients
                     'ret_nrel',                                         # Retreatment without relapse
                     'c_newinc',                                         # New cases  
                     'newrel_f04', 'newrel_f59',  'newrel_f1014',
                     'newrel_f1519', 'newrel_f2024', 'newrel_f2534',
                     'newrel_f3544', 'newrel_f4554', 'newrel_f5564',
                     'newrel_f65',
                     'newrel_m04', 'newrel_m59',  'newrel_m1014',
                     'newrel_m1519', 'newrel_m2024', 'newrel_m2534',
                     'newrel_m3544', 'newrel_m4554', 'newrel_m5564',
                     'newrel_m65',
                     'ret_rel_labconf', 'ret_rel_clindx', 'ret_rel_ep',
                     'ret_nrel', 'c_newinc', 'newrel_hivpos', 
                     'newrel_art'
                     ]
TB_NOTIF_DIST_KEYS = ['new_labconf_perc', 'ret_rel_labconf_perc']


TB_RUN_DEFAULT_COUNTRIES = None#("ZWE", )

# "ALB",
# "ASM",
# "AND",
# "ABW",
# "BHR",
# "BEL",
# "BMU",
# "VGB",
# "BRN",
# "CYM",
# "COK",
# "CUW",
# "CYP",
# "PRK",
# "GRD",
# "JOR",
# "KWT",
# "LVA",
# "LUX",
# "MDV",
# "MLT",
# "MCO",
# "MNE",
# "MSR",
# "NRU",
# "NIU",
# "NOR",
# "PLW",
# "PRT",
# "QAT",
# "KOR",
# "KNA",
# "SMR",
# "SYC",
# "SGP",
# "SXM",
# "SVK",
# "SVN",
# "SSD",
# "TON",
# "TKM",
# "TCA",
# "TUV",
# "WLF" 
# )


import numpy as np

import AvenirCommon.Util.Util as acu

import SpectrumCommon.Const.GB.GBConst as gbc
import SpectrumCommon.Const.PC.PCConst as pcc
import SpectrumCommon.Const.PC.PCConstLink as pccl
import SpectrumCommon.Const.PC.PCModVarFields as pcmvf
import SpectrumCommon.Util.PC.PCUtilLink as pcul
import SpectrumCommon.Modvars.PC.PCUtil as pcu

#######################################################################################################
#                                                                                                     #
#                   Area data record - contains basically everything in PC for a                      #
#                   particular programme area, delivery channel, package, or national programme       #
#                                                                                                     #
#######################################################################################################

def PC_create_area_data_dict(area_type_mst_ID: int, ID: str, num_years: int, costing_mode: int):
    num_rows = pcc.PC_NUM_STANDARD_ROWS

    if (costing_mode == gbc.GB_IHT_COSTING_MODE) and (ID == str(pccl.PC_IH_HIV_AIDS_MST_ID)):
        num_rows = pcc.PC_TOTAL_NUM_HIV_ROWS

    return {
        pcmvf.PC_AREA_TYPE_MST_ID : area_type_mst_ID,
        pcmvf.PC_AREA_ID          : ID,
        pcmvf.PC_MAIN_ROW_COSTS   : np.full((num_rows, num_years), 0.0).tolist(),
    }


# def init_area_dict_list(area_ID_list: list, num_years: int):
#     area_dict_list = []

#     for area_ID in area_ID_list:
#         area_data_dict = create_area_data_dict(area_ID, num_years)
#         area_dict_list.append(area_data_dict)

#     return area_dict_list


def PC_init_area_dict_list(area_type_mst_ID: int, prog_area_mod_var_dict_list: list, num_years: int, costing_mode: int):
    area_dict_list = []

    if area_type_mst_ID == pccl.PC_IH_PROG_AREA_MST_ID:
        for area_mod_var_dict in prog_area_mod_var_dict_list:
            prog_area_ID = pcul.PC_get_IH_prog_area_ID(area_mod_var_dict)
            area_data_dict = PC_create_area_data_dict(area_type_mst_ID, prog_area_ID, num_years, costing_mode)
            area_dict_list.append(area_data_dict)

    return area_dict_list


def PC_create_prog_mgmt_data_dict(mod_mst_ID: int, num_years: int):
    num_rows = pcc.PC_NUM_STANDARD_ROWS

    if mod_mst_ID != gbc.GB_IS:
        num_rows = pcc.PC_PM_NUM_STANDARD_ROWS
    else:
        num_rows = pcc.PC_PM_TOTAL_NUM_IS_ROWS

    return {
        pcmvf.PC_MAIN_ROW_COSTS: np.full((num_rows, num_years), 0.0).tolist(),
    }

#######################################################################################################
#                                                                                                     #
#                                     Cost category records                                           #
#                                                                                                     #
#######################################################################################################

def PC_create_cost_cat_rec(ID: str, name: str, string_const: str):

    return {
        pcmvf.PC_COST_CAT_ID         : ID,
        pcmvf.PC_COST_CAT_NAME       : name,
        pcmvf.PC_COST_CAT_STR_CONST  : string_const,
        pcmvf.PC_COST_CAT_CUSTOM     : False,
    }

#######################################################################################################
#                                                                                                     #
#                                              Cost input records                                     #
#                                                                                                     #
#######################################################################################################

def PC_create_cost_input_rec(ID: str, name: str, cost_cat_ID: int):

    return {
        pcmvf.PC_COST_INPUT_ID                        : ID,
        pcmvf.PC_COST_INPUT_NAME                      : name,
        pcmvf.PC_COST_INPUT_STR_CONST                 : '',
        pcmvf.PC_COST_INPUT_CAT_ID                    : cost_cat_ID,
        pcmvf.PC_COST_INPUT_CUSTOM                    : False,
        pcmvf.PC_COST_INPUT_NOTE                      : '',
        pcmvf.PC_COST_INPUT_SOURCE                    : '',
        pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE           : '',
        pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST : '',
        pcmvf.PC_COST_INPUT_UNIT_COST                 : 0.0,
    }

#######################################################################################################
#                                                                                                     #
#                                 Costing section and costing subsection records                      #
#                                                                                                     #
#######################################################################################################

def PC_create_costing_section_rec(area_type_ID: int, area_ID: str, ID: str, name: str, string_const: str, num_years: int, costing_mode: int):

    costing_section_rec = {
        pcmvf.PC_COSTING_SECT_AREA_TYPE_ID    : area_type_ID,
        pcmvf.PC_COSTING_SECT_AREA_ID         : area_ID,
        pcmvf.PC_COSTING_SECT_ID              : ID,
        pcmvf.PC_COSTING_SECT_NAME            : name,
        pcmvf.PC_COSTING_SECT_STR_CONST       : string_const,
        pcmvf.PC_COSTING_SECT_CUSTOM          : False,
    }

    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        costing_section_rec[pcmvf.PC_COSTING_SECT_ENTRY_MODE_CS] = gbc.GB_PERCENT
        costing_section_rec[pcmvf.PC_COSTING_SECT_COSTS_CS] = np.full((num_years), 0.0).tolist()
    else:
        costing_section_rec[pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS] = []

    return costing_section_rec

# Area ID should be the programme area (currently just NCD)
def PC_create_costing_section_rec_PPLI(area_ID: str, ID: str, name: str, string_const: str, num_years: int):
    rec = PC_create_costing_section_rec(pccl.PC_IH_PPLI_MST_ID, area_ID, ID, name, string_const, num_years, gbc.GB_IHT_COSTING_MODE)

    # Taxation tab (For Tobacco control, alcohol control, sugar-sweetened beverage control sections); Will vary a bit for alcohol and beverages though,
    # So only do this for Tobacco control for now...

    # Upper table
    rec[pcmvf.PC_COSTING_SECT_PRICE_ELASTICITY_PPLI] = np.full((pcc.PC_NUM_TAX_COLS_PPLI), 0.0).tolist()
    rec[pcmvf.PC_COSTING_SECT_INCOME_ELASTICITY_PPLI] = np.full((pcc.PC_NUM_TAX_COLS_PPLI), 0.0).tolist()
    rec[pcmvf.PC_COSTING_SECT_PRICE_INCOME_ONLY_PPLI] = np.full((pcc.PC_NUM_TAX_COLS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_NON_PRICE_MEASURES_PPLI] = np.full((pcc.PC_NUM_TAX_COLS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_ALL_MPOWER_PPLI] = np.full((pcc.PC_NUM_TAX_COLS_PPLI), 0).tolist()

    # Lower table
    rec[pcmvf.PC_COSTING_SECT_TAX_ENTRY_MODE_PPLI] = pcc.PC_ENTER_REAL_RETAIL_PRICE_PPLI_MST_ID
    rec[pcmvf.PC_COSTING_SECT_REAL_RETAIL_PRICE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_EXCISE_SHARE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_TAX_SHARE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_REAL_EXCISE_PER_PACK_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_REAL_TAX_PER_PACK_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_REAL_INDUSTRY_PRICE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_TAX_ADJUST_CONSUMP_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_EXCISE_REVENUE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_TAX_REVENUE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()
    rec[pcmvf.PC_COSTING_SECT_USAGE_RATE_PPLI] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0.0).tolist()
    rec[pcmvf.PC_COSTING_SECT_NUM_USERS] = np.full((pcc.PC_NUM_YEAR_ENDPOINTS_PPLI), 0).tolist()

    return rec

def PC_create_costing_subsection_rec(ID: str, name: str, string_const: str):

    return {
        pcmvf.PC_COSTING_SUBSECT_ID        : ID,
        pcmvf.PC_COSTING_SUBSECT_NAME      : name,
        pcmvf.PC_COSTING_SUBSECT_STR_CONST : string_const,
        pcmvf.PC_COSTING_SUBSECT_CUSTOM    : False,
    }

def PC_create_costing_subsection_rec_PPLI(ID: str, name: str, string_const: str, num_years: int):
    rec = PC_create_costing_subsection_rec(ID, name, string_const)

    rec[pcmvf.PC_COSTING_SUBSECT_ADMIN_UNITS_PER_LEVEL_OPTION_PPLI] = pcc.PC_USE_NUM_ADMIN_UNITS_FROM_CONFIG_PPLI_MST_ID
    rec[pcmvf.PC_COSTING_SUBSECT_ADMIN_UNITS_STD_POP_SIZE_PPLI] = np.full((pcc.PC_PPLI_NUM_ADMIN_LEVELS), 0).tolist()
    rec[pcmvf.PC_COSTING_SUBSECT_ENTER_TOTAL_INPUTS_PER_ADMIN_REG_YEAR_PPLI] = False

    if ID == pcc.PC_RAISE_TAXES_PPLI_COSTING_SUBSECT_ID:
        rec[pcmvf.PC_COSTING_SUBSECT_RAISE_TAXES_PRICE_PPLI] = np.full((num_years), 0).tolist()
        rec[pcmvf.PC_COSTING_SUBSECT_RAISE_TAXES_EXCISE_SHARE_PPLI] = np.full((num_years), 0).tolist()
        rec[pcmvf.PC_COSTING_SUBSECT_RAISE_TAXES_TAX_SHARE_PPLI] = np.full((num_years), 0).tolist()

    phase_intensity_values = np.full((pcc.PC_NUM_PHASE_INTENSITY_ROW_TYPES_PPLI, num_years), 0)
    phase_intensity_values[pcc.PC_PHASE_ROW_PPLI_CURR_ID - 1, :] = pcc.PC_NO_ACTION_PHASE_0_PPLI_MST_ID
    rec[pcmvf.PC_COSTING_SUBSECT_PHASE_INTENSITY_VALUES_PPLI] = phase_intensity_values.tolist()

    return rec
    
#######################################################################################################
#                                                                                                     #
#                                       Activity records                                              #
#                                                                                                     #
#######################################################################################################

def PC_create_activity_rec(area_type_ID: int, area_ID: str, costing_sect_ID: str, costing_subsection_ID: str, activity_ID: str, name: str):

    return {
        pcmvf.PC_ACT_AREA_TYPE_ID       : area_type_ID,
        pcmvf.PC_ACT_AREA_ID            : area_ID,
        pcmvf.PC_ACT_COSTING_SECT_ID    : costing_sect_ID,
        pcmvf.PC_ACT_COSTING_SUBSECT_ID : costing_subsection_ID,
        pcmvf.PC_ACT_ID                 : activity_ID,
        pcmvf.PC_ACT_CUSTOM             : False,
        pcmvf.PC_ACT_NAME               : name,
        pcmvf.PC_ACT_STR_CONST          : '',
    }

#######################################################################################################
#                                                                                                     #
#                                      Activity line item (ALI) records                               #
#                                                                                                     #
#######################################################################################################

def PC_create_activity_line_item_rec(area_type_ID: int, area_ID: str, costing_sect_ID: str, costing_subsect_ID: str, act_ID: str, cost_input_ID: str, num_years: int, pop_and_policy_level_intervs: bool):

    rec = {
        pcmvf.PC_ALI_AREA_TYPE_ID        : area_type_ID, 
        pcmvf.PC_ALI_AREA_ID             : area_ID,
        pcmvf.PC_ALI_COSTING_SECT_ID     : costing_sect_ID, 
        pcmvf.PC_ALI_COSTING_SUBSECT_ID  : costing_subsect_ID,
        pcmvf.PC_ALI_ACT_ID              : act_ID,
        pcmvf.PC_ALI_COST_INPUT_ID       : cost_input_ID,
    }

    if not pop_and_policy_level_intervs:
        rec[pcmvf.PC_ALI_FREQUENCY] = np.full((num_years), 0.0).tolist()
        # We'll use these fields for PPLI too, but they need to be by admin level in that case
        rec[pcmvf.PC_ALI_NUM_UNITS] = 0
        rec[pcmvf.PC_ALI_DURATION] = 0

    return rec

# Area ID should be the programme area (currently just NCD)
def PC_create_activity_line_item_rec_PPLI(area_ID: str, costing_sect_ID: str, costing_subsect_ID: str, act_ID: str, cost_input_ID: str, num_years: int):
    rec = PC_create_activity_line_item_rec(pccl.PC_IH_PPLI_MST_ID, area_ID, costing_sect_ID, costing_subsect_ID, act_ID, cost_input_ID, num_years, True)

    # Each field needs to be specified for Inputs per admin level and Total inputs (the former feeds the latter and the latter is what is used by results)
    rec[pcmvf.PC_ALI_QUANTITY_PER_ADMIN_REG_YEAR_PPLI] = np.full((pcc.PC_NUM_PPLI_ALI_SETS, pcc.PC_PPLI_NUM_ADMIN_LEVELS, num_years), 0.0).tolist()
    rec[pcmvf.PC_ALI_NUM_UNITS] = np.full((pcc.PC_PPLI_NUM_ADMIN_LEVELS), 0.0).tolist()
    rec[pcmvf.PC_ALI_DURATION] = np.full((pcc.PC_PPLI_NUM_ADMIN_LEVELS), 0.0).tolist()

    return rec

def PC_create_costs_by_sect_rec(area_type_ID: str, area_ID: str, costing_sect_ID: str,  value: list):

    return {
        pcmvf.PC_F_AREA_TYPE_ID        : area_type_ID,
        pcmvf.PC_F_AREA_ID             : area_ID,
        pcmvf.PC_F_COSTING_SECT_ID     : costing_sect_ID,
        pcmvf.PC_F_VALUE               : value,
    }

def PC_create_costs_by_subsect_rec(area_type_ID: str, area_ID: str, costing_sect_ID: str, costing_subsect_ID: str, value: list):

    return {
        pcmvf.PC_F_AREA_TYPE_ID        : area_type_ID,
        pcmvf.PC_F_AREA_ID             : area_ID,
        pcmvf.PC_F_COSTING_SECT_ID     : costing_sect_ID,
        pcmvf.PC_F_COSTING_SUBSECT_ID  : costing_subsect_ID,
        pcmvf.PC_F_VALUE               : value,
    }

def PC_create_costs_by_act_rec(area_type_ID: str, area_ID: str, costing_sect_ID: str,  costing_subsect_ID: str, act_ID: str, value: list):

    return {
        pcmvf.PC_F_AREA_TYPE_ID        : area_type_ID,
        pcmvf.PC_F_AREA_ID             : area_ID,
        pcmvf.PC_F_COSTING_SECT_ID     : costing_sect_ID,
        pcmvf.PC_F_COSTING_SUBSECT_ID  : costing_subsect_ID,
        pcmvf.PC_F_ACT_ID              : act_ID,
        pcmvf.PC_F_VALUE               : value,
    }

def PC_create_costs_by_act_and_cost_cat_rec(area_type_ID: str, area_ID: str, costing_sect_ID: str, costing_subsect_ID: str, act_ID: str, cost_cat_ID: str, value: list):

    return {
        pcmvf.PC_F_AREA_TYPE_ID        : area_type_ID,
        pcmvf.PC_F_AREA_ID             : area_ID,
        pcmvf.PC_F_COSTING_SECT_ID     : costing_sect_ID,
        pcmvf.PC_F_COSTING_SUBSECT_ID  : costing_subsect_ID,
        pcmvf.PC_F_ACT_ID              : act_ID,
        pcmvf.PC_F_COST_CAT_ID         : cost_cat_ID,
        pcmvf.PC_F_VALUE               : value,
    }

########################################################################################################################################
########################################################################################################################################
#                                                                                                                                      #
#                                                       Meta data                                                                      #
#                                                                                                                                      #
########################################################################################################################################
########################################################################################################################################

def PC_init_row_info_meta(prog_area_ID):
    row_info_meta = []

    if prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID:
        num_rows = pcc.PC_TOTAL_NUM_HIV_ROWS
    else:
        num_rows = pcc.PC_NUM_STANDARD_ROWS

    for row in acu.GBRange(1, num_rows):
        mst_ID = pcu.get_PC_row_mst_ID(prog_area_ID, row)

        row_info_meta.append(
            {
                "mstID"          : mst_ID,
                "name"           : pcu.get_PC_row_name(mst_ID),
                "stringConstant" : pcu.get_PC_row_string_constant(mst_ID),
                "isAggregate"    : pcu.get_PC_row_is_aggregate(mst_ID),
            }
        )

        # Some programme areas have rows that are not costed (ex: coverage rows for HIV). Add a field so we can mark which rows are costs.
        if prog_area_ID == pccl.PC_IH_HIV_AIDS_MST_ID:
            row_info_meta[row - 1]["isCost"] = pcu.get_PC_row_is_cost(prog_area_ID, mst_ID)
            row_info_meta[row - 1]["singleLine"] = pcu.get_PC_row_is_single_line(prog_area_ID, mst_ID)
            row_info_meta[row - 1]["hasData"] = pcu.get_PC_row_has_data(prog_area_ID, mst_ID)

    return row_info_meta




from SpectrumCommon.Const.FP import *

fp_age_indices = {
    'All Ages' : FP_All_Ages,
    'Age 15 to 19'   : FP_A15_19,
    'Age 20 to 24'   : FP_A20_24,
    'Age 25 to 29'   : FP_A25_29,
    'Age 30 to 34'   : FP_A30_34,
    'Age 35 to 39'   : FP_A35_39,
    'Age 40 to 44'   : FP_A40_44,
    'Age 45 to 49'   : FP_A45_49,
}

fp_method_indices = {
  #'All methods' : FP_all_methods ,
  'Condom(Female)' : FP_CondomFM,
  'Condom(Male)' : FP_CondomML,
  'Female sterilization' : FP_fster,
  'Male sterilization' : FP_mster,
  'Vaginal barrier' : FP_vag_barrier,
  'VFT' : FP_VFT,
  'Withdrawal' : FP_withdrawal,
  'Periodic Abstinance' : FP_PerAbst,
  'Traditional' : FP_Traditional,
  'Injectable 3 month' : FP_inj3month,
  'Injectable 2 month' : FP_inj2month,
  'Injectable 1 month' : FP_inj1month,
  'Injectable 6 month' : FP_inj6month,
  'Uninject' : FP_Uninject,
  'Implanon' : FP_Implanon,
  'Jadelle' : FP_Jadelle,
  'Sino implant' : FP_Sino_Implant,
  'Standard Pill' : FP_StandardPill,
  'Progestin' : FP_Progestin,
  'Peri coital' : FP_Peri_coital ,
  'IUD Copper' : FP_IUDCopper,
  'IUD LngIus' : FP_IUDLngIus,
  'LAM' : FP_LAM,
  'SDM' : FP_SDM,
  'other' : FP_other,
  'custom methods':f'{FP_LastDefMethod+1} - {FP_Max_Methods}'
}

fp_all_method_indices ={
    'All methods' : FP_all_methods,
}
fp_all_method_indices.update(fp_method_indices)


fp_need_indices = {
 'All need' : FP_all_need,
 'Spacing ' : FP_spacing,
 'Limiting' : FP_limiting
}

fp_child_survival_indices = {
    'Per birth risk'          : FP_PerBirthRisk,
    'Fertility Adjusted IMR'  : FP_FertAdjIMR,
    'Fertility Adjusted U5MR' : FP_FertAdjU5MR,
    'Relative risky birth'    : FP_RelRiskyBirth,
    'Relative IMR'            : FP_RelIMR,
    'Relative U5MR'           : FP_RelU5MR      
}

# PAC Input Constants 
fp_pac_inputs_indices = {
    'PerAbortLegal'           : FP_PerAbortLegal,
    'PerIllAbortTreat'        : FP_PerIllAbortTreat,
    'PerLegalAbortTreat'      : FP_PerLegalAbortTreat,
    'MMRMaternDeath'          : FP_MMRMaternDeath,
    'PerMaternDeathAbortion'  : FP_PerMaternDeathAbortion ,
    'MortalityUntreatVsTreat' : FP_MortalityUntreatVsTreat,
    'AnnPostAbortCare'        : FP_AnnPostAbortCare,
    'CostAbortCompTreat'      : FP_CostAbortCompTreat,
    'CostCounServCase'        : FP_CostCounServCase,
}
fp_pac_outputs_indices = {

    'Wanted pregnancies'    : FP_WantedPreg,
    'Unwanted pregnancies'  : FP_UnwantedPreg,
    'Per abortion need treatment received' : FP_PerAbortNeedTreatReceive,
    'MMR births one' : FP_MMRBirthsOne,
    'Abortion deaths per abortion' : FP_AbortDeathsPerAbort,
    'Death treat Ill Abort      ' : FP_DeathTreatIllAbort,
    'Death untreat Ill Abort    ' : FP_DeathUntreatIllAbort,
    'MD Total          ' : FP_MDTotal,
    'MD WantBirths     ' : FP_MDWantBirths,
    'MD UnwantBirths   ' : FP_MDUnwantBirths, 
    'MD TreatIllAbort  ' : FP_MDTreatIllAbort, 
    'MD UntreatIllAbort' : FP_MDUntreatIllAbort, 
    'TotDeathAvert' : FP_TotDeathAvert, 
    'Births' : FP_Births, 
    'WantedBirths' : FP_WantedBirths, 
    'UnwantedBirths' : FP_UnwantedBirths , 
    'Abortions' : FP_Abortions, 
    'IllAbortions' : FP_IllAbortions, 
    'IllAbortNoTreat' : FP_IllAbortNoTreat, 
    'IllAbortTreat' : FP_IllAbortTreat, 
    'TreatIllAbort' : FP_TreatIllAbort, 
    'UntreatIllAbort' : FP_UntreatIllAbort, 
    'MMR' : FP_MMR, 
    'MMRBirthsTwo' : FP_MMRBirthsTwo, 
    'MMRAbort' : FP_MMRAbort, 
    'MaternDeathAbort' : FP_MaternDeathAbort, 
    'LivesSaved' : FP_LivesSaved, 
    'CostTreatUnsafeAbort' : FP_CostTreatUnsafeAbort   , 
    'CostPerMaternDeathAvert ' : FP_CostPerMaternDeathAvert, 
}

fp_lam_indices = {
    'LAM 0 to 1' : FP_LAM0_1,
    'LAM 2 to 3' : FP_LAM2_3,
    'LAM 4 to 5' : FP_LAM4_5,
}

# Pregnancy constants 
fp_pregnancies_indices = {
    'All pregnancies' : FP_all_PREG,
    'Wanted pregnancies' : FP_wanted,
    'Unwanted pregnancies' : FP_unwanted,
    'Mistimed pregnancies' : FP_mistimed
}

fp_rr_birth_order_indices = {  
    'First birth'            : FP_RRFirstBirth,
    'Second to third births' : FP_RRSecToThirdBirth,
    'More than four births'  : FP_RRMoreFourBirth
}
fp_rr_age_order_indices = {
    'Less than 18'   : FP_RRLess18,
    '18 to 34'       : FP_RR18To34,
    'More than 34'   : FP_RRMore34
}
fp_rr_birth_interval_indices = {
    # 'First birth'            : FP_RRFirstBirth, 
    'Less 18 Months'     : FP_RRLess18Months,
    '18 to 23Months'     : FP_RR18to23Months,
    'More than 24Months'     : FP_RRMore24Months
}
fp_rr_advance_indices = {
    
    'RR Intercept' : FP_RRIntrcept,
    'RR C1'        : FP_RRC1,
    'RR C2'        : FP_RRC2 
}

fp_rr_mothers_age_indices = {
    'RR birth order greater than 3' : FP_RRBOGreater3,
    'RR mothers age Greater than 35'  : FP_RRMAGreat35,
    'RR mothers age 18 to 34'   : FP_RRMA18To34 
}